class Cud():

    def __init__(self):
        #print("__init__ called")
        pass

    def __repr__(self):
        print("__repr__ called")
        return ""

    def __lt__(self, other):
        print("__lt__ called")

    def __le__(self, other):
        print("__le__ called")

    def __eq__(self, other):
        print("__eq__ called")

    def __ne__(self, other):
        print("__ne__ called")

    def __ge__(self, other):
        print("__ge__ called")

    def __gt__(self, other):
        print("__gt__ called")

    def __abs__(self):
        print("__abs__ called")

    def __add__(self, other):
        print("__add__ called")

    def __and__(self, other):
        print("__and__ called")

    def __floordiv__(self, other):
        print("__floordiv__ called")

    def __invert__(self):
        print("__invert__ called")

    def __lshift__(self, val):
        print("__lshift__ called")

    def __mod__(self, val):
        print("__mod__ called")

    def __mul__(self, other):
        print("__mul__ called")

    def __matmul__(self, other):
        print("__matmul__ called")

    def __neg__(self):
        print("__neg__ called")

    def __or__(self, other):
        print("__or__ called")

    def __pos__(self):
        print("__pos__ called")

    def __pow__(self, val):
        print("__pow__ called")

    def __rshift__(self, val):
        print("__rshift__ called")

    def __sub__(self, other):
        print("__sub__ called")

    def __truediv__(self, other):
        print("__truediv__ called")

    def __div__(self, other):
        print("__div__ called")

    def __xor__(self, other):
        print("__xor__ called")

    def __iadd__(self, other):
        print("__iadd__ called")
        return self

    def __isub__(self, other):
        print("__isub__ called")
        return self

    def __dir__(self):
        return ['a', 'b', 'c']

cud1 = Cud()
cud2 = Cud()

try:
    +cud1
except TypeError:
    print("SKIP")
    raise SystemExit

# the following require MICROPY_PY_ALL_SPECIAL_METHODS
+cud1
-cud1
~cud1
cud1 * cud2
cud1 @ cud2
cud1 / cud2
cud2 // cud1
cud1 += cud2
cud1 -= cud2
cud1 % 2
cud1 ** 2
cud1 | cud2
cud1 & cud2
cud1 ^ cud2
cud1 << 1
cud1 >> 1

# test that dir() delegates to __dir__ special method
print(dir(cud1))

# test that dir() does not delegate to __dir__ for the type
print('a' in dir(Cud))



# Simple test of a TCP server and client transferring data

import socket

PORT = 8000


# Server
def instance0():
    multitest.globals(IP=multitest.get_network_ip())
    s = socket.socket()
    s.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
    s.bind(socket.getaddrinfo("0.0.0.0", PORT)[0][-1])
    s.listen()
    multitest.next()
    s2, _ = s.accept()
    print(s2.recv(16))
    s2.send(b"server to client")
    s2.close()
    s.close()


# Client
def instance1():
    multitest.next()
    s = socket.socket()
    s.connect(socket.getaddrinfo(IP, PORT)[0][-1])
    s.send(b"client to server")
    print(s.recv(16))
    s.close()



# MicroPython asyncio module
# MIT license; Copyright (c) 2019-2020 Damien P. George

from . import core


class Stream:
    def __init__(self, s, e={}):
        self.s = s
        self.e = e
        self.out_buf = b""

    def get_extra_info(self, v):
        return self.e[v]

    def close(self):
        pass

    async def wait_closed(self):
        # TODO yield?
        self.s.close()

    # async
    def read(self, n=-1):
        r = b""
        while True:
            yield core._io_queue.queue_read(self.s)
            r2 = self.s.read(n)
            if r2 is not None:
                if n >= 0:
                    return r2
                if not len(r2):
                    return r
                r += r2

    # async
    def readinto(self, buf):
        yield core._io_queue.queue_read(self.s)
        return self.s.readinto(buf)

    # async
    def readexactly(self, n):
        r = b""
        while n:
            yield core._io_queue.queue_read(self.s)
            r2 = self.s.read(n)
            if r2 is not None:
                if not len(r2):
                    raise EOFError
                r += r2
                n -= len(r2)
        return r

    # async
    def readline(self):
        l = b""
        while True:
            yield core._io_queue.queue_read(self.s)
            l2 = self.s.readline()  # may do multiple reads but won't block
            if l2 is None:
                continue
            l += l2
            if not l2 or l[-1] == 10:  # \n (check l in case l2 is str)
                return l

    def write(self, buf):
        if not self.out_buf:
            # Try to write immediately to the underlying stream.
            ret = self.s.write(buf)
            if ret == len(buf):
                return
            if ret is not None:
                buf = buf[ret:]
        self.out_buf += buf

    # async
    def drain(self):
        if not self.out_buf:
            # Drain must always yield, so a tight loop of write+drain can't block the scheduler.
            return (yield from core.sleep_ms(0))
        mv = memoryview(self.out_buf)
        off = 0
        while off < len(mv):
            yield core._io_queue.queue_write(self.s)
            ret = self.s.write(mv[off:])
            if ret is not None:
                off += ret
        self.out_buf = b""


# Stream can be used for both reading and writing to save code size
StreamReader = Stream
StreamWriter = Stream


# Create a TCP stream connection to a remote host
#
# async
def open_connection(host, port, ssl=None, server_hostname=None):
    from errno import EINPROGRESS
    import socket

    ai = socket.getaddrinfo(host, port, 0, socket.SOCK_STREAM)[0]  # TODO this is blocking!
    s = socket.socket(ai[0], ai[1], ai[2])
    s.setblocking(False)
    try:
        s.connect(ai[-1])
    except OSError as er:
        if er.errno != EINPROGRESS:
            raise er
    # wrap with SSL, if requested
    if ssl:
        if ssl is True:
            import ssl as _ssl

            ssl = _ssl.SSLContext(_ssl.PROTOCOL_TLS_CLIENT)
        if not server_hostname:
            server_hostname = host
        s = ssl.wrap_socket(s, server_hostname=server_hostname, do_handshake_on_connect=False)
        s.setblocking(False)
    ss = Stream(s)
    yield core._io_queue.queue_write(s)
    return ss, ss


# Class representing a TCP stream server, can be closed and used in "async with"
class Server:
    async def __aenter__(self):
        return self

    async def __aexit__(self, exc_type, exc, tb):
        self.close()
        await self.wait_closed()

    def close(self):
        # Note: the _serve task must have already started by now due to the sleep
        # in start_server, so `state` won't be clobbered at the start of _serve.
        self.state = True
        self.task.cancel()

    async def wait_closed(self):
        await self.task

    async def _serve(self, s, cb, ssl):
        self.state = False
        # Accept incoming connections
        while True:
            try:
                yield core._io_queue.queue_read(s)
            except core.CancelledError as er:
                # The server task was cancelled, shutdown server and close socket.
                s.close()
                if self.state:
                    # If the server was explicitly closed, ignore the cancellation.
                    return
                else:
                    # Otherwise e.g. the parent task was cancelled, propagate
                    # cancellation.
                    raise er
            try:
                s2, addr = s.accept()
            except:
                # Ignore a failed accept
                continue
            if ssl:
                try:
                    s2 = ssl.wrap_socket(s2, server_side=True, do_handshake_on_connect=False)
                except OSError as e:
                    core.sys.print_exception(e)
                    s2.close()
                    continue
            s2.setblocking(False)
            s2s = Stream(s2, {"peername": addr})
            core.create_task(cb(s2s, s2s))


# Helper function to start a TCP stream server, running as a new task
# TODO could use an accept-callback on socket read activity instead of creating a task
async def start_server(cb, host, port, backlog=5, ssl=None):
    import socket

    # Create and bind server socket.
    host = socket.getaddrinfo(host, port)[0]  # TODO this is blocking!
    s = socket.socket()
    s.setblocking(False)
    s.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
    s.bind(host[-1])
    s.listen(backlog)

    # Create and return server object and task.
    srv = Server()
    srv.task = core.create_task(srv._serve(s, cb, ssl))
    try:
        # Ensure that the _serve task has been scheduled so that it gets to
        # handle cancellation.
        await core.sleep_ms(0)
    except core.CancelledError as er:
        # If the parent task is cancelled during this first sleep, then
        # we will leak the task and it will sit waiting for the socket, so
        # cancel it.
        srv.task.cancel()
        raise er
    return srv


################################################################################
# Legacy uasyncio compatibility


async def stream_awrite(self, buf, off=0, sz=-1):
    if off != 0 or sz != -1:
        buf = memoryview(buf)
        if sz == -1:
            sz = len(buf)
        buf = buf[off : off + sz]
    self.write(buf)
    await self.drain()


Stream.aclose = Stream.wait_closed
Stream.awrite = stream_awrite
Stream.awritestr = stream_awrite  # TODO explicitly convert to bytes?



# Test select.poll in combination with file descriptors.

try:
    import select, errno

    select.poll  # Raises AttributeError for CPython implementations without poll()
except (ImportError, AttributeError):
    print("SKIP")
    raise SystemExit

# Check that poll supports registering file descriptors (integers).
try:
    select.poll().register(0)
except OSError:
    print("SKIP")
    raise SystemExit

# Register invalid file descriptor.
try:
    select.poll().register(-1)
except ValueError:
    print("ValueError")

# Test polling stdout, it should be writable.
poller = select.poll()
poller.register(1)
poller.modify(1, select.POLLOUT)
print(poller.poll())

# Unregister then re-register.
poller.unregister(1)
poller.register(1, select.POLLIN)

# Poll for input, should return an empty list.
print(poller.poll(0))

# Test registering a very large number of file descriptors (will trigger
# EINVAL due to more than OPEN_MAX fds). Typically it's 1024 (and on GitHub CI
# we force this via `ulimit -n 1024`).
poller = select.poll()
for fd in range(6000):
    poller.register(fd)
try:
    poller.poll()
    assert False
except OSError as er:
    print(er.errno == errno.EINVAL)

# Register stdout/stderr, plus many extra ones to trigger the fd vector
# resizing. Then unregister the excess ones and verify poll still works.
poller = select.poll()
for fd in range(1, 1000):
    poller.register(fd)
for i in range(3, 1000):
    poller.unregister(i)
print(sorted(poller.poll()))



# test pyb module on F405 MCUs

import os, pyb

if not "STM32F405" in os.uname().machine:
    print("SKIP")
    raise SystemExit

print(pyb.freq())
print(type(pyb.rng()))

# test HAL error specific to F405
i2c = pyb.I2C(2, pyb.I2C.CONTROLLER)
try:
    i2c.recv(1, 1)
except OSError as e:
    print(repr(e))



# This example demonstrates a peripheral implementing the Nordic UART Service (NUS).

# This example demonstrates the low-level bluetooth module. For most
# applications, we recommend using the higher-level aioble library which takes
# care of all IRQ handling and connection management. See
# https://github.com/micropython/micropython-lib/tree/master/micropython/bluetooth/aioble

import bluetooth
from ble_advertising import advertising_payload

from micropython import const

_IRQ_CENTRAL_CONNECT = const(1)
_IRQ_CENTRAL_DISCONNECT = const(2)
_IRQ_GATTS_WRITE = const(3)

_FLAG_WRITE = const(0x0008)
_FLAG_NOTIFY = const(0x0010)

_UART_UUID = bluetooth.UUID("6E400001-B5A3-F393-E0A9-E50E24DCCA9E")
_UART_TX = (
    bluetooth.UUID("6E400003-B5A3-F393-E0A9-E50E24DCCA9E"),
    _FLAG_NOTIFY,
)
_UART_RX = (
    bluetooth.UUID("6E400002-B5A3-F393-E0A9-E50E24DCCA9E"),
    _FLAG_WRITE,
)
_UART_SERVICE = (
    _UART_UUID,
    (_UART_TX, _UART_RX),
)

# org.bluetooth.characteristic.gap.appearance.xml
_ADV_APPEARANCE_GENERIC_COMPUTER = const(128)


class BLEUART:
    def __init__(self, ble, name="mpy-uart", rxbuf=100):
        self._ble = ble
        self._ble.active(True)
        self._ble.irq(self._irq)
        ((self._tx_handle, self._rx_handle),) = self._ble.gatts_register_services((_UART_SERVICE,))
        # Increase the size of the rx buffer and enable append mode.
        self._ble.gatts_set_buffer(self._rx_handle, rxbuf, True)
        self._connections = set()
        self._rx_buffer = bytearray()
        self._handler = None
        # Optionally add services=[_UART_UUID], but this is likely to make the payload too large.
        self._payload = advertising_payload(name=name, appearance=_ADV_APPEARANCE_GENERIC_COMPUTER)
        self._advertise()

    def irq(self, handler):
        self._handler = handler

    def _irq(self, event, data):
        # Track connections so we can send notifications.
        if event == _IRQ_CENTRAL_CONNECT:
            conn_handle, _, _ = data
            self._connections.add(conn_handle)
        elif event == _IRQ_CENTRAL_DISCONNECT:
            conn_handle, _, _ = data
            if conn_handle in self._connections:
                self._connections.remove(conn_handle)
            # Start advertising again to allow a new connection.
            self._advertise()
        elif event == _IRQ_GATTS_WRITE:
            conn_handle, value_handle = data
            if conn_handle in self._connections and value_handle == self._rx_handle:
                self._rx_buffer += self._ble.gatts_read(self._rx_handle)
                if self._handler:
                    self._handler()

    def any(self):
        return len(self._rx_buffer)

    def read(self, sz=None):
        if not sz:
            sz = len(self._rx_buffer)
        result = self._rx_buffer[0:sz]
        self._rx_buffer = self._rx_buffer[sz:]
        return result

    def write(self, data):
        for conn_handle in self._connections:
            self._ble.gatts_notify(conn_handle, self._tx_handle, data)

    def close(self):
        for conn_handle in self._connections:
            self._ble.gap_disconnect(conn_handle)
        self._connections.clear()

    def _advertise(self, interval_us=500000):
        self._ble.gap_advertise(interval_us, adv_data=self._payload)


def demo():
    import time

    ble = bluetooth.BLE()
    uart = BLEUART(ble)

    def on_rx():
        print("rx: ", uart.read().decode().strip())

    uart.irq(handler=on_rx)
    nums = [4, 8, 15, 16, 23, 42]
    i = 0

    try:
        while True:
            uart.write(str(nums[i]) + "\n")
            i = (i + 1) % len(nums)
            time.sleep_ms(1000)
    except KeyboardInterrupt:
        pass

    uart.close()


if __name__ == "__main__":
    demo()



# Test for sdcard block protocol
# Peter hinch 30th Jan 2016
import machine
import os
import sdcard


def sdtest():
    spi = machine.SPI(1)
    spi.init()  # Ensure right baudrate
    sd = sdcard.SDCard(spi, machine.Pin.board.X21)  # Compatible with PCB
    vfs = os.VfsFat(sd)
    os.mount(vfs, "/fc")
    print("Filesystem check")
    print(os.listdir("/fc"))

    line = "abcdefghijklmnopqrstuvwxyz\n"
    lines = line * 200  # 5400 chars
    short = "1234567890\n"

    fn = "/fc/rats.txt"
    print()
    print("Multiple block read/write")
    with open(fn, "w") as f:
        n = f.write(lines)
        print(n, "bytes written")
        n = f.write(short)
        print(n, "bytes written")
        n = f.write(lines)
        print(n, "bytes written")

    with open(fn, "r") as f:
        result1 = f.read()
        print(len(result1), "bytes read")

    fn = "/fc/rats1.txt"
    print()
    print("Single block read/write")
    with open(fn, "w") as f:
        n = f.write(short)  # one block
        print(n, "bytes written")

    with open(fn, "r") as f:
        result2 = f.read()
        print(len(result2), "bytes read")

    os.umount("/fc")

    print()
    print("Verifying data read back")
    success = True
    if result1 == "".join((lines, short, lines)):
        print("Large file Pass")
    else:
        print("Large file Fail")
        success = False
    if result2 == short:
        print("Small file Pass")
    else:
        print("Small file Fail")
        success = False
    print()
    print("Tests", "passed" if success else "failed")



# cmdline: -v -v
# test printing of all bytecodes
# fmt: off

def f():
    # constants
    a = None + False + True
    a = 0
    a = 1000
    a = -1000

    # constructing data
    a = 1
    b = (1, 2)
    c = [1, 2]
    d = {1, 2}
    e = {}
    f = {1:2}
    g = 'a'
    h = b'a'

    # unary/binary ops
    i = 1
    j = 2
    k = a + b
    l = -a
    m = not a
    m = a == b == c
    m = not (a == b and b == c)

    # attributes
    n = b.c
    b.c = n

    # subscript
    p = b[0]
    b[0] = p
    b[0] += p

    # slice
    a = b[::]

    # sequence unpacking
    a, b = c
    a, *a = a

    # tuple swapping
    a, b = b, a
    a, b, c = c, b, a

    # del fast
    del a

    # globals
    global gl
    gl = a
    del gl

    # comprehensions
    a = (b for c in d if e)
    a = [b for c in d if e]
    a = {b:b for c in d if e}

    # function calls
    a()
    a(1)
    a(b=1)
    a(*b)

    # method calls
    a.b()
    a.b(1)
    a.b(c=1)
    a.b(*c)

    # jumps
    if a:
        x
    else:
        y
    while a:
        b
    while not a:
        b
    a = a or a

    # for loop
    for a in b:
        c

    # exceptions
    try:
        while a:
            break
    except:
        b
    finally:
        c
    while a:
        try:
            break
        except:
            pass

    # with
    with a:
        b

    # closed over variables
    x = 1
    def closure():
        nonlocal x; a = x + 1
        x = 1
        del x

    # import
    import a
    from a import b
    #from sys import * # tested at module scope

    # raise
    if a: raise
    if a: raise 1

    # return
    if a: return
    if a: return 1

# function with lots of locals
def f():
    l1 = l2 = l3 = l4 = l5 = l6 = l7 = l8 = l9 = l10 = 1
    m1 = m2 = m3 = m4 = m5 = m6 = m7 = m8 = m9 = m10 = 2
    l10 + m10

# functions with default args
def f(a=1):
    pass

    def f(b=2):
        return b + a

# function which yields
def f():
    yield
    yield 1
    yield from 1

# class
class Class:
    pass

# delete name
del Class

# load super method
def f(self):
    super().f()

# import * (needs to be in module scope)
from sys import *



# yield-from a user-defined generator with a throw() method

class Iter:
    def __iter__(self):
        return self

    def __next__(self):
        return 1

    def throw(self, x):
        print('throw', x)
        return 456

def gen():
    yield from Iter()

# calling close() should not call throw()
g = gen()
print(next(g))
g.close()

# can throw a non-exception object
g = gen()
print(next(g))
print(g.throw(123))

# throwing an exception class just injects that class
g = gen()
print(next(g))
print(g.throw(ZeroDivisionError))

# this user-defined generator doesn't have a throw() method
class Iter2:
    def __iter__(self):
        return self

    def __next__(self):
        return 1

def gen2():
    yield from Iter2()

# the thrown ValueError is not intercepted by the user class
g = gen2()
print(next(g))
try:
    g.throw(ValueError)
except:
    print('ValueError')

# the thrown 123 is not an exception so raises a TypeError
g = gen2()
print(next(g))
try:
    g.throw(123)
except TypeError:
    print('TypeError')



class C1:
    def __call__(self, val):
        print('call', val)
        return 'item'

class C2:

    def __getattr__(self, k):
        pass

c1 = C1()
print(c1(1))

c2 = C2()
try:
    print(c2(1))
except TypeError:
    print("TypeError")



# test array + array
try:
    import array
except ImportError:
    print("SKIP")
    raise SystemExit

a1 = array.array('I', [1])
a2 = array.array('I', [2])
print(a1 + a2)

a1 += array.array('I', [3, 4])
print(a1)

a1.extend(array.array('I', [5]))
print(a1)



import unittest


class TestUnittestSetup(unittest.TestCase):
    class_setup_var = 0

    @classmethod
    def setUpClass(cls):
        assert cls is TestUnittestSetup
        TestUnittestSetup.class_setup_var += 1

    @classmethod
    def tearDownClass(cls):
        assert cls is TestUnittestSetup
        # Not sure how to actually test this, but we can check (in the test case below)
        # that it hasn't been run already at least.
        TestUnittestSetup.class_setup_var = -1

    def testSetUpTearDownClass_1(self):
        assert TestUnittestSetup.class_setup_var == 1, TestUnittestSetup.class_setup_var

    def testSetUpTearDownClass_2(self):
        # Test this twice, as if setUpClass() gets run like setUp() it would be run twice
        assert TestUnittestSetup.class_setup_var == 1, TestUnittestSetup.class_setup_var


if __name__ == "__main__":
    unittest.main()



import os
import tempfile
import unittest


class Base(unittest.TestCase):
    def assertExists(self, fn):
        os.stat(fn)

    def assertNotExists(self, fn):
        with self.assertRaises(OSError):
            os.stat(fn)


class TestMkdtemp(Base):
    def test_no_args(self):
        fn = tempfile.mkdtemp()
        self.assertTrue(fn.startswith("/tmp"))
        self.assertExists(fn)
        os.rmdir(fn)

    def test_prefix(self):
        fn = tempfile.mkdtemp(prefix="foo")
        self.assertTrue(fn.startswith("/tmp"))
        self.assertTrue("foo" in fn)
        self.assertFalse(fn.endswith("foo"))
        self.assertExists(fn)
        os.rmdir(fn)

    def test_suffix(self):
        fn = tempfile.mkdtemp(suffix="foo")
        self.assertTrue(fn.startswith("/tmp"))
        self.assertTrue(fn.endswith("foo"))
        self.assertExists(fn)
        os.rmdir(fn)

    def test_dir(self):
        fn = tempfile.mkdtemp(dir="tmp_micropython")
        self.assertTrue(fn.startswith("tmp_micropython"))
        self.assertExists(fn)
        os.rmdir(fn)


class TestTemporaryDirectory(Base):
    def test_context_manager_no_args(self):
        with tempfile.TemporaryDirectory() as fn:
            self.assertTrue(isinstance(fn, str))
            self.assertTrue(fn.startswith("/tmp"))
            self.assertExists(fn)
        self.assertNotExists(fn)



# Test VfsFat class and its finaliser

try:
    import errno, os, vfs

    vfs.VfsFat
except (ImportError, AttributeError):
    print("SKIP")
    raise SystemExit


class RAMBlockDevice:
    def __init__(self, blocks, sec_size=512):
        self.sec_size = sec_size
        self.data = bytearray(blocks * self.sec_size)

    def readblocks(self, n, buf):
        for i in range(len(buf)):
            buf[i] = self.data[n * self.sec_size + i]

    def writeblocks(self, n, buf):
        for i in range(len(buf)):
            self.data[n * self.sec_size + i] = buf[i]

    def ioctl(self, op, arg):
        if op == 4:  # MP_BLOCKDEV_IOCTL_BLOCK_COUNT
            return len(self.data) // self.sec_size
        if op == 5:  # MP_BLOCKDEV_IOCTL_BLOCK_SIZE
            return self.sec_size


# Create block device, and skip test if not enough RAM
try:
    import errno, os

    bdev = RAMBlockDevice(50)
except MemoryError:
    print("SKIP")
    raise SystemExit

# Format block device and create VFS object
vfs.VfsFat.mkfs(bdev)
fs = vfs.VfsFat(bdev)

# Here we test that opening a file with the heap locked fails correctly.  This
# is a special case because file objects use a finaliser and allocating with a
# finaliser is a different path to normal allocation.  It would be better to
# test this in the core tests but there are no core objects that use finaliser.
import micropython

micropython.heap_lock()
try:
    import errno, os

    fs.open("x", "r")
except MemoryError:
    print("MemoryError")
micropython.heap_unlock()

# Here we test that the finaliser is actually called during a garbage collection.
import gc

# Preallocate global variables, and list of filenames for the test (which may
# in turn allocate new qstrs and/or a new qstr pool).
f = None
n = None
names = ["x%d" % i for i in range(5)]

# Do a large number of single-block allocations to move the GC head forwards,
# ensuring that the files are allocated from never-before-used blocks and
# therefore couldn't possibly have any references to them left behind on
# the stack.
for i in range(1024):
    []

# Run the test: create files without closing them, run GC, then read back files.
# Only read back N-1 files because the last one may not be finalised due to
# references to it being left on the C stack.
for n in names:
    f = fs.open(n, "w")
    f.write(n)
    f = None  # release f without closing
gc.collect()  # should finalise at least the first N-1 files by closing them
for n in names[:-1]:
    with fs.open(n, "r") as f:
        print(f.read())



from pyb import Servo

servo = Servo(1)
print(servo)

servo.angle(0)
servo.angle(10, 100)

servo.speed(-10)
servo.speed(10, 100)

servo.pulse_width(1500)
print(servo.pulse_width())

servo.calibration(630, 2410, 1490, 2460, 2190)
print(servo.calibration())



"""
categories: Core,Runtime
description: Code running in eval() function doesn't have access to local variables
cause: MicroPython doesn't maintain symbolic local environment, it is optimized to an array of slots. Thus, local variables can't be accessed by a name. Effectively, ``eval(expr)`` in MicroPython is equivalent to ``eval(expr, globals(), globals())``.
workaround: Unknown
"""
val = 1


def test():
    val = 2
    print(val)
    eval("print(val)")


test()



# Test ssl.wrap_socket() with cadata passed in.

try:
    import io
    import ssl
except ImportError:
    print("SKIP")
    raise SystemExit

# Invalid cadata.
try:
    ssl.wrap_socket(io.BytesIO(), cadata=b"!")
except AttributeError:
    # "cadata" keyword argument is not supported by axtls.
    print("SKIP")
    raise SystemExit
except ValueError as er:
    print(repr(er))



# test _thread.get_ident() function
#
# MIT license; Copyright (c) 2016 Damien P. George on behalf of Pycom Ltd

import _thread


# Initialise variables (also preallocate their spot in the globals dict so the
# globals dict is not resized while threads are running).
tid = None
tid_main = None
new_tid = None
finished = False


def thread_entry():
    global tid
    tid = _thread.get_ident()
    print("thread", type(tid) == int, tid != 0, tid != tid_main)
    global finished
    finished = True


tid_main = _thread.get_ident()
print("main", type(tid_main) == int, tid_main != 0)

new_tid = _thread.start_new_thread(thread_entry, ())

while not finished:
    pass

print("done", type(new_tid) == int, new_tid == tid)



# Test reverse operators.

# Test user type with integers.
class A:
    def __init__(self, v):
        self.v = v

    def __add__(self, o):
        if isinstance(o, A):
            return A(self.v + o.v)
        return A(self.v + o)

    def __radd__(self, o):
        return A(self.v + o)

    def __repr__(self):
        return "A({})".format(self.v)


print(A(3) + 1)
print(2 + A(5))


# Test user type with strings.
class B:
    def __init__(self, v):
        self.v = v

    def __repr__(self):
        return "B({})".format(self.v)

    def __ror__(self, o):
        return B(o + "|" + self.v)

    def __radd__(self, o):
        return B(o + "+" + self.v)

    def __rmul__(self, o):
        return B(o + "*" + self.v)

    def __rtruediv__(self, o):
        return B(o + "/" + self.v)


print("a" | B("b"))
print("a" + B("b"))
print("a" * B("b"))
print("a" / B("b"))

x = "a"; x |= B("b"); print(x)
x = "a"; x += B("b"); print(x)
x = "a"; x *= B("b"); print(x)
x = "a"; x /= B("b"); print(x)



class defaultdict:
    @staticmethod
    def __new__(cls, default_factory=None, **kwargs):
        # Some code (e.g. urllib.urlparse) expects that basic defaultdict
        # functionality will be available to subclasses without them
        # calling __init__().
        self = super(defaultdict, cls).__new__(cls)
        self.d = {}
        return self

    def __init__(self, default_factory=None, **kwargs):
        self.d = kwargs
        self.default_factory = default_factory

    def __getitem__(self, key):
        try:
            return self.d[key]
        except KeyError:
            v = self.__missing__(key)
            self.d[key] = v
            return v

    def __setitem__(self, key, v):
        self.d[key] = v

    def __delitem__(self, key):
        del self.d[key]

    def __contains__(self, key):
        return key in self.d

    def __missing__(self, key):
        if self.default_factory is None:
            raise KeyError(key)
        return self.default_factory()



#!/usr/bin/env python3
#
# This file is part of the MicroPython project, http://micropython.org/
#
# The MIT License (MIT)
#
# Copyright (c) 2019 Damien P. George
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.

from __future__ import print_function
import sys
import os
import subprocess

# Always use the mpy-cross from this repo.
sys.path.insert(0, os.path.join(os.path.dirname(__file__), "../mpy-cross"))
import mpy_cross

import manifestfile

VARS = {}


class FreezeError(Exception):
    pass


def system(cmd):
    try:
        output = subprocess.check_output(cmd, stderr=subprocess.STDOUT)
        return 0, output
    except subprocess.CalledProcessError as er:
        return -1, er.output


def get_timestamp(path, default=None):
    try:
        stat = os.stat(path)
        return stat.st_mtime
    except OSError:
        if default is None:
            raise FreezeError("cannot stat {}".format(path))
        return default


def mkdir(filename):
    path = os.path.dirname(filename)
    if not os.path.isdir(path):
        os.makedirs(path)


# Formerly make-frozen.py.
# This generates:
# - MP_FROZEN_STR_NAMES macro
# - mp_frozen_str_sizes
# - mp_frozen_str_content
def generate_frozen_str_content(modules):
    output = [
        b"#include <stdint.h>\n",
        b"#define MP_FROZEN_STR_NAMES \\\n",
    ]

    for _, target_path in modules:
        print("STR", target_path)
        output.append(b'"%s\\0" \\\n' % target_path.encode())
    output.append(b"\n")

    output.append(b"const uint32_t mp_frozen_str_sizes[] = { ")

    for full_path, _ in modules:
        st = os.stat(full_path)
        output.append(b"%d, " % st.st_size)
    output.append(b"0 };\n")

    output.append(b"const char mp_frozen_str_content[] = {\n")
    for full_path, _ in modules:
        with open(full_path, "rb") as f:
            data = f.read()

            # We need to properly escape the script data to create a C string.
            # When C parses hex characters of the form \x00 it keeps parsing the hex
            # data until it encounters a non-hex character.  Thus one must create
            # strings of the form "data\x01" "abc" to properly encode this kind of
            # data.  We could just encode all characters as hex digits but it's nice
            # to be able to read the resulting C code as ASCII when possible.

            data = bytearray(data)  # so Python2 extracts each byte as an integer
            esc_dict = {ord("\n"): b"\\n", ord("\r"): b"\\r", ord('"'): b'\\"', ord("\\"): b"\\\\"}
            output.append(b'"')
            break_str = False
            for c in data:
                try:
                    output.append(esc_dict[c])
                except KeyError:
                    if 32 <= c <= 126:
                        if break_str:
                            output.append(b'" "')
                            break_str = False
                        output.append(chr(c).encode())
                    else:
                        output.append(b"\\x%02x" % c)
                        break_str = True
            output.append(b'\\0"\n')

    output.append(b'"\\0"\n};\n\n')
    return b"".join(output)


def main():
    # Parse arguments
    import argparse

    cmd_parser = argparse.ArgumentParser(
        description="A tool to generate frozen content in MicroPython firmware images."
    )
    cmd_parser.add_argument("-o", "--output", help="output path")
    cmd_parser.add_argument("-b", "--build-dir", help="output path")
    cmd_parser.add_argument(
        "-f", "--mpy-cross-flags", default="", help="flags to pass to mpy-cross"
    )
    cmd_parser.add_argument("-v", "--var", action="append", help="variables to substitute")
    cmd_parser.add_argument("--mpy-tool-flags", default="", help="flags to pass to mpy-tool")
    cmd_parser.add_argument("files", nargs="+", help="input manifest list")
    args = cmd_parser.parse_args()

    # Extract variables for substitution.
    for var in args.var:
        name, value = var.split("=", 1)
        if os.path.exists(value):
            value = os.path.abspath(value)
        VARS[name] = value

    if "MPY_DIR" not in VARS or "PORT_DIR" not in VARS:
        print("MPY_DIR and PORT_DIR variables must be specified")
        sys.exit(1)

    # Get paths to tools
    MPY_CROSS = VARS["MPY_DIR"] + "/mpy-cross/build/mpy-cross"
    if sys.platform == "win32":
        MPY_CROSS += ".exe"
    MPY_CROSS = os.getenv("MICROPY_MPYCROSS", MPY_CROSS)
    MPY_TOOL = VARS["MPY_DIR"] + "/tools/mpy-tool.py"

    # Ensure mpy-cross is built
    if not os.path.exists(MPY_CROSS) and not os.path.exists(MPY_CROSS + ".exe"):
        print("mpy-cross not found at {}, please build it first".format(MPY_CROSS))
        sys.exit(1)

    manifest = manifestfile.ManifestFile(manifestfile.MODE_FREEZE, VARS)

    # Include top-level inputs, to generate the manifest
    for input_manifest in args.files:
        try:
            manifest.execute(input_manifest)
        except manifestfile.ManifestFileError as er:
            print('freeze error executing "{}": {}'.format(input_manifest, er.args[0]))
            sys.exit(1)

    # Process the manifest
    str_paths = []
    mpy_files = []
    ts_newest = 0
    for result in manifest.files():
        if result.kind == manifestfile.KIND_FREEZE_AS_STR:
            str_paths.append(
                (
                    result.full_path,
                    result.target_path,
                )
            )
            ts_outfile = result.timestamp
        elif result.kind == manifestfile.KIND_FREEZE_AS_MPY:
            outfile = "{}/frozen_mpy/{}.mpy".format(args.build_dir, result.target_path[:-3])
            ts_outfile = get_timestamp(outfile, 0)
            if result.timestamp >= ts_outfile:
                print("MPY", result.target_path)
                mkdir(outfile)
                # Add __version__ to the end of the file before compiling.
                with manifestfile.tagged_py_file(result.full_path, result.metadata) as tagged_path:
                    try:
                        mpy_cross.compile(
                            tagged_path,
                            dest=outfile,
                            src_path=result.target_path,
                            opt=result.opt,
                            mpy_cross=MPY_CROSS,
                            extra_args=args.mpy_cross_flags.split(),
                        )
                    except mpy_cross.CrossCompileError as ex:
                        print("error compiling {}:".format(result.target_path))
                        print(ex.args[0])
                        raise SystemExit(1)
                ts_outfile = get_timestamp(outfile)
            mpy_files.append(outfile)
        else:
            assert result.kind == manifestfile.KIND_FREEZE_MPY
            mpy_files.append(result.full_path)
            ts_outfile = result.timestamp
        ts_newest = max(ts_newest, ts_outfile)

    # Check if output file needs generating
    if ts_newest < get_timestamp(args.output, 0):
        # No files are newer than output file so it does not need updating
        return

    # Freeze paths as strings
    output_str = generate_frozen_str_content(str_paths)

    # Freeze .mpy files
    if mpy_files:
        res, output_mpy = system(
            [
                sys.executable,
                MPY_TOOL,
                "-f",
                "-q",
                args.build_dir + "/genhdr/qstrdefs.preprocessed.h",
            ]
            + args.mpy_tool_flags.split()
            + mpy_files
        )
        if res != 0:
            print("error freezing mpy {}:".format(mpy_files))
            print(output_mpy.decode())
            sys.exit(1)
    else:
        output_mpy = (
            b'#include "py/emitglue.h"\n'
            b"extern const qstr_pool_t mp_qstr_const_pool;\n"
            b"const qstr_pool_t mp_qstr_frozen_const_pool = {\n"
            b"    #if MICROPY_QSTR_BYTES_IN_HASH\n"
            b"    (qstr_pool_t*)&mp_qstr_const_pool, MP_QSTRnumber_of, 0, 0, 0, NULL, NULL, {},\n"
            b"    #else\n"
            b"    (qstr_pool_t*)&mp_qstr_const_pool, MP_QSTRnumber_of, 0, 0, 0, NULL, {},\n"
            b"    #endif\n"
            b"};\n"
            b'const char mp_frozen_names[] = { MP_FROZEN_STR_NAMES "\\0"};\n'
            b"const mp_raw_code_t *const mp_frozen_mpy_content[] = {NULL};\n"
        )

    # Generate output
    print("GEN", args.output)
    mkdir(args.output)
    with open(args.output, "wb") as f:
        f.write(b"//\n// Content for MICROPY_MODULE_FROZEN_STR\n//\n")
        f.write(output_str)
        f.write(b"//\n// Content for MICROPY_MODULE_FROZEN_MPY\n//\n")
        f.write(output_mpy)


if __name__ == "__main__":
    main()



try:
    from collections import OrderedDict
    import uctypes
except ImportError:
    print("SKIP")
    raise SystemExit

desc = OrderedDict(
    {
        # arr is array at offset 0, of UINT8 elements, array size is 2
        "arr": (uctypes.ARRAY | 0, uctypes.UINT8 | 2),
        # arr2 is array at offset 0, size 2, of structures defined recursively
        "arr2": (uctypes.ARRAY | 0, 2, {"b": uctypes.UINT8 | 0}),
        "arr3": (uctypes.ARRAY | 2, uctypes.UINT16 | 2),
        "arr4": (uctypes.ARRAY | 0, 2, {"b": uctypes.UINT8 | 0, "w": uctypes.UINT16 | 1}),
        "sub": (
            0,
            {
                "b1": uctypes.BFUINT8 | 0 | 4 << uctypes.BF_POS | 4 << uctypes.BF_LEN,
                "b2": uctypes.BFUINT8 | 0 | 0 << uctypes.BF_POS | 4 << uctypes.BF_LEN,
            },
        ),
    }
)

data = bytearray(b"01234567")

S = uctypes.struct(uctypes.addressof(data), desc, uctypes.LITTLE_ENDIAN)

print(uctypes.sizeof(S.arr))
assert uctypes.sizeof(S.arr) == 2

print(uctypes.sizeof(S.arr2))
assert uctypes.sizeof(S.arr2) == 2

print(uctypes.sizeof(S.arr3))

try:
    print(uctypes.sizeof(S.arr3[0]))
except TypeError:
    print("TypeError")

print(uctypes.sizeof(S.arr4))
assert uctypes.sizeof(S.arr4) == 6

print(uctypes.sizeof(S.sub))
assert uctypes.sizeof(S.sub) == 1



# test calling builtin import function

# basic test
__import__("builtins")

# first arg should be a string
try:
    __import__(1)
except TypeError:
    print("TypeError")

# module name should not be empty
try:
    __import__("")
except ValueError:
    print("ValueError")

# level argument should be non-negative
try:
    __import__("xyz", None, None, None, -1)
except ValueError:
    print("ValueError")



import hmac

# Uncomment to use micropython-lib hashlib (supports sha512)
# import sys
# sys.path.append('../hashlib')

import hashlib

msg = b"zlutoucky kun upel dabelske ody"

dig = hmac.new(b"1234567890", msg=msg, digestmod="sha256").hexdigest()

print("c735e751e36b08fb01e25794bdb15e7289b82aecdb652c8f4f72f307b39dad39")
print(dig)

if dig != "c735e751e36b08fb01e25794bdb15e7289b82aecdb652c8f4f72f307b39dad39":
    raise Exception("Error")

if hasattr(hashlib, "sha512"):
    dig = hmac.new(b"1234567890", msg=msg, digestmod=hashlib.sha512).hexdigest()

    print(
        "59942f31b6f5473fb4eb630fabf5358a49bc11d24ebc83b114b4af30d6ef47ea14b673f478586f520a0b9c53b27c8f8dd618c165ef586195bd4e98293d34df1a"
    )
    print(dig)

    if (
        dig
        != "59942f31b6f5473fb4eb630fabf5358a49bc11d24ebc83b114b4af30d6ef47ea14b673f478586f520a0b9c53b27c8f8dd618c165ef586195bd4e98293d34df1a"
    ):
        raise Exception("Error")
else:
    print("sha512 not supported")

key = b"\x06\x1au\x90|Xz;o\x1b<\xafGL\xbfn\x8a\xc94YPfC^\xb9\xdd)\x7f\xaf\x85\xa1\xed\x82\xbexp\xaf\x13\x1a\x9d"

dig = hmac.new(key[:20], msg=msg, digestmod=hashlib.sha256).hexdigest()

print("59e332b881df09fdecf569c8b142b27fc989638720aeda2813f82442b6e3d91b")
print(dig)

if dig != "59e332b881df09fdecf569c8b142b27fc989638720aeda2813f82442b6e3d91b":
    raise Exception("Error")

dig = hmac.new(key[:32], msg=msg, digestmod=hashlib.sha256).hexdigest()

print("b72fed815cd71acfa3a2f5cf2343679565fa18e7cd92226ab443aabd1fd7b7b0")
print(dig)

if dig != "b72fed815cd71acfa3a2f5cf2343679565fa18e7cd92226ab443aabd1fd7b7b0":
    raise Exception("Error")

dig = hmac.new(key, msg=msg, digestmod=hashlib.sha256).hexdigest()

print("4e51beae6c2b0f90bb3e99d8e93a32d168b6c1e9b7d2130e2d668a3b3e10358d")
print(dig)

if dig != "4e51beae6c2b0f90bb3e99d8e93a32d168b6c1e9b7d2130e2d668a3b3e10358d":
    raise Exception("Error")



# This file is part of the MicroPython project, http://micropython.org/
#
# The MIT License (MIT)
#
# Copyright (c) 2020-2021 Damien P. George
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
"""
Utility to create compressed, encrypted and signed DFU files.
"""

import argparse
import os
import re
import struct
import sys
import zlib

sys.path.append(os.path.dirname(__file__) + "/../../../tools")
import dfu

try:
    import pyhy
except ImportError:
    raise SystemExit(
        "ERROR: pyhy not found. Please install python pyhy for encrypted mboot support: pip3 install pyhy"
    )


# Currently supported version of a packed DFU file.
MBOOT_PACK_HEADER_VERSION = 1

# Must match MBOOT_PACK_HYDRO_CONTEXT in mboot/pack.h
MBOOT_PACK_HYDRO_CONTEXT = "mbootenc"

# Must match enum in mboot/pack.h.
MBOOT_PACK_CHUNK_META = 0
MBOOT_PACK_CHUNK_FULL_SIG = 1
MBOOT_PACK_CHUNK_FW_RAW = 2
MBOOT_PACK_CHUNK_FW_GZIP = 3


class Keys:
    def __init__(self, filename):
        self.filename = filename

    def generate(self):
        kp = pyhy.hydro_sign_keygen()
        self.sign_sk = kp.sk
        self.sign_pk = kp.pk
        self.secretbox = pyhy.hydro_secretbox_keygen()

    def _save_data(self, name, data, file_, hide=False):
        prefix = "//" if hide else ""
        data = ",".join("0x{:02x}".format(b) for b in data)
        file_.write("{}const uint8_t {}[] = {{{}}};\n".format(prefix, name, data))

    def _load_data(self, name, line):
        line = line.split(name + "[] = ")
        if len(line) != 2:
            raise Exception("malformed input keys: {}".format(line))
        data = line[1].strip()
        return bytes(int(value, 16) for value in data[1:-2].split(","))

    def save(self):
        with open(self.filename, "w") as f:
            self._save_data("mboot_pack_sign_secret_key", self.sign_sk, f, hide=True)
            self._save_data("mboot_pack_sign_public_key", self.sign_pk, f)
            self._save_data("mboot_pack_secretbox_key", self.secretbox, f)

    def load(self):
        with open(self.filename) as f:
            for line in f:
                for key, attr in (
                    ("mboot_pack_sign_secret_key", "sign_sk"),
                    ("mboot_pack_sign_public_key", "sign_pk"),
                    ("mboot_pack_secretbox_key", "secretbox"),
                ):
                    if key in line:
                        setattr(self, attr, self._load_data(key, line))


def dfu_read(filename):
    elems = []

    with open(filename, "rb") as f:
        hdr = f.read(11)
        sig, ver, size, num_targ = struct.unpack("<5sBIB", hdr)
        file_offset = 11

        for i in range(num_targ):
            hdr = f.read(274)
            sig, alt, has_name, name, t_size, num_elem = struct.unpack("<6sBi255sII", hdr)

            file_offset += 274
            file_offset_t = file_offset
            for j in range(num_elem):
                hdr = f.read(8)
                addr, e_size = struct.unpack("<II", hdr)
                data = f.read(e_size)
                elems.append((addr, data))
                file_offset += 8 + e_size

            if t_size != file_offset - file_offset_t:
                raise Exception("corrupt DFU {} {}".format(t_size, file_offset - file_offset_t))

        if size != file_offset:
            raise Exception("corrupt DFU {} {}".format(size, file_offset))

        hdr = f.read(16)
        hdr = struct.unpack("<HHHH3sBI", hdr)
        vid_pid = "0x{:04x}:0x{:04x}".format(hdr[2], hdr[1])

    return vid_pid, elems


def compress(data):
    c = zlib.compressobj(level=9, memLevel=9, wbits=-15)  # wsize=15, no header
    return c.compress(data) + c.flush()


def encrypt(keys, data):
    return pyhy.hydro_secretbox_encrypt(data, 0, MBOOT_PACK_HYDRO_CONTEXT, keys.secretbox)


def sign(keys, data):
    if not hasattr(keys, "sign_sk"):
        raise Exception("packing a dfu requires a secret key")
    return pyhy.hydro_sign_create(data, MBOOT_PACK_HYDRO_CONTEXT, keys.sign_sk)


def pack_chunk(keys, format_, chunk_addr, chunk_payload):
    header = struct.pack(
        "<BBBBII", MBOOT_PACK_HEADER_VERSION, format_, 0, 0, chunk_addr, len(chunk_payload)
    )
    chunk = header + chunk_payload
    sig = sign(keys, chunk)
    chunk = chunk + sig
    return chunk


def data_chunks(data, n):
    for i in range(0, len(data), n):
        yield data[i : i + n]


def generate_keys(keys, args):
    keys.generate()
    keys.save()


def pack_dfu(keys, args):
    chunk_size = int(args.chunk_size[0])

    # Load previously generated keys.
    keys.load()

    # Read the input DFU file.
    vid_pid, elems = dfu_read(args.infile[0])

    # Ensure firmware sections are processed in order of destination memory address.
    elems = sorted(elems, key=lambda e: e[0])

    # Build list of packed chunks.
    target = []
    full_fw = b""
    full_signature_payload = b""
    for address, fw in elems:
        # Update full firmware and full signature chunk.
        full_fw += fw
        full_signature_payload += struct.pack("<II", address, len(fw))

        # Split the firmware into chunks, encrypt and sign the chunks
        # then register them as individual DFU targets.
        for i, chunk in enumerate(data_chunks(fw, chunk_size)):
            chunk_addr = address + i * chunk_size
            if args.gzip:
                chunk = compress(chunk)
            chunk = encrypt(keys, chunk)
            chunk = pack_chunk(
                keys,
                MBOOT_PACK_CHUNK_FW_GZIP if args.gzip else MBOOT_PACK_CHUNK_FW_RAW,
                chunk_addr,
                chunk,
            )
            target.append({"address": chunk_addr, "data": chunk})

    # Add full signature to targets, at location following the last chunk.
    chunk_addr += chunk_size
    sig = sign(keys, full_fw)
    full_signature_payload += sig
    full_signature_chunk = pack_chunk(
        keys, MBOOT_PACK_CHUNK_FULL_SIG, chunk_addr, full_signature_payload
    )
    target.append({"address": chunk_addr, "data": full_signature_chunk})

    # Build the packed DFU file of all the encrypted and signed chunks.
    dfu.build(args.outfile[0], [target], vid_pid)

    # Verify the packed DFU file.
    verify_pack_dfu(keys, args.outfile[0])


def verify_pack_dfu(keys, filename):
    """Verify packed dfu file against keys. Gathers decrypted binary data."""
    full_sig = pyhy.hydro_sign(MBOOT_PACK_HYDRO_CONTEXT)
    _, elems = dfu_read(filename)
    base_addr = None
    binary_data = b""

    for addr, data in elems:
        if base_addr is None:
            base_addr = addr

        header = struct.unpack("<BBBBII", data[:12])
        chunk = data[12 : 12 + header[5]]
        sig = data[12 + header[5] :]
        sig_pass = pyhy.hydro_sign_verify(
            sig, data[:12] + chunk, MBOOT_PACK_HYDRO_CONTEXT, keys.sign_pk
        )
        assert sig_pass

        if header[1] == MBOOT_PACK_CHUNK_FULL_SIG:
            actual_sig = chunk[-64:]
        else:
            chunk = pyhy.hydro_secretbox_decrypt(
                chunk, 0, MBOOT_PACK_HYDRO_CONTEXT, keys.secretbox
            )
            assert chunk is not None
            if header[1] == MBOOT_PACK_CHUNK_FW_GZIP:
                chunk = zlib.decompress(chunk, wbits=-15)
            full_sig.update(chunk)
            assert addr == base_addr + len(binary_data)
            binary_data += chunk

    full_sig_pass = full_sig.final_verify(actual_sig, keys.sign_pk)
    assert full_sig_pass
    return [{"address": base_addr, "data": binary_data}]


def unpack_dfu(keys, args):
    # Load previously generated keys.
    keys.load()

    # Build a DFU file from the decrypted binary data.
    data = verify_pack_dfu(keys, args.infile[0])
    dfu.build(args.outfile[0], [data])


def main():
    cmd_parser = argparse.ArgumentParser(description="Build signed/encrypted DFU files")
    cmd_parser.add_argument("-k", "--keys", default="mboot_keys.h", help="filename for keys")
    subparsers = cmd_parser.add_subparsers()

    parser_gk = subparsers.add_parser("generate-keys", help="generate keys")
    parser_gk.set_defaults(func=generate_keys)

    parser_ed = subparsers.add_parser("pack-dfu", help="encrypt and sign a DFU file")
    parser_ed.add_argument("-z", "--gzip", action="store_true", help="compress chunks")
    parser_ed.add_argument("chunk_size", nargs=1, help="maximum size in bytes of each chunk")
    parser_ed.add_argument("infile", nargs=1, help="input DFU file")
    parser_ed.add_argument("outfile", nargs=1, help="output DFU file")
    parser_ed.set_defaults(func=pack_dfu)

    parser_dd = subparsers.add_parser("unpack-dfu", help="decrypt a signed/encrypted DFU file")
    parser_dd.add_argument("infile", nargs=1, help="input packed DFU file")
    parser_dd.add_argument("outfile", nargs=1, help="output DFU file")
    parser_dd.set_defaults(func=unpack_dfu)

    args = cmd_parser.parse_args()

    keys = Keys(args.keys)
    args.func(keys, args)


if __name__ == "__main__":
    main()



# MicroPython USB host library, compatible with PyUSB.
# MIT license; Copyright (c) 2021-2024 Damien P. George

import usb.control


def claim_interface(device, interface):
    device._claim_interface(interface)


def get_string(device, index):
    bs = usb.control.get_descriptor(device, 254, 3, index, 0)
    s = ""
    for i in range(2, bs[0] & 0xFE, 2):
        s += chr(bs[i] | bs[i + 1] << 8)
    return s



# MicroPython USB CDC module
# MIT license; Copyright (c) 2022 Martin Fischer, 2023-2024 Angus Gratton
import io
import time
import errno
import machine
import struct
from micropython import const

from .core import Interface, Buffer, split_bmRequestType

_EP_IN_FLAG = const(1 << 7)

# Control transfer stages
_STAGE_IDLE = const(0)
_STAGE_SETUP = const(1)
_STAGE_DATA = const(2)
_STAGE_ACK = const(3)

# Request types
_REQ_TYPE_STANDARD = const(0x0)
_REQ_TYPE_CLASS = const(0x1)
_REQ_TYPE_VENDOR = const(0x2)
_REQ_TYPE_RESERVED = const(0x3)

_DEV_CLASS_MISC = const(0xEF)
_CS_DESC_TYPE = const(0x24)  # CS Interface type communication descriptor

# CDC control interface definitions
_INTERFACE_CLASS_CDC = const(2)
_INTERFACE_SUBCLASS_CDC = const(2)  # Abstract Control Mode
_PROTOCOL_NONE = const(0)  # no protocol

# CDC descriptor subtype
# see also CDC120.pdf, table 13
_CDC_FUNC_DESC_HEADER = const(0)
_CDC_FUNC_DESC_CALL_MANAGEMENT = const(1)
_CDC_FUNC_DESC_ABSTRACT_CONTROL = const(2)
_CDC_FUNC_DESC_UNION = const(6)

# CDC class requests, table 13, PSTN subclass
_SET_LINE_CODING_REQ = const(0x20)
_GET_LINE_CODING_REQ = const(0x21)
_SET_CONTROL_LINE_STATE = const(0x22)
_SEND_BREAK_REQ = const(0x23)

_LINE_CODING_STOP_BIT_1 = const(0)
_LINE_CODING_STOP_BIT_1_5 = const(1)
_LINE_CODING_STOP_BIT_2 = const(2)

_LINE_CODING_PARITY_NONE = const(0)
_LINE_CODING_PARITY_ODD = const(1)
_LINE_CODING_PARITY_EVEN = const(2)
_LINE_CODING_PARITY_MARK = const(3)
_LINE_CODING_PARITY_SPACE = const(4)

_LINE_STATE_DTR = const(1)
_LINE_STATE_RTS = const(2)

_PARITY_BITS_REPR = "NOEMS"
_STOP_BITS_REPR = ("1", "1.5", "2")

# Other definitions
_CDC_VERSION = const(0x0120)  # release number in binary-coded decimal

# Number of endpoints in each interface
_CDC_CONTROL_EP_NUM = const(1)
_CDC_DATA_EP_NUM = const(2)

# CDC data interface definitions
_CDC_ITF_DATA_CLASS = const(0xA)
_CDC_ITF_DATA_SUBCLASS = const(0)
_CDC_ITF_DATA_PROT = const(0)  # no protocol

# Length of the bulk transfer endpoints. Maybe should be configurable?
_BULK_EP_LEN = const(64)

# MicroPython error constants (negated as IOBase.ioctl uses negative return values for error codes)
# these must match values in py/mperrno.h
_MP_EINVAL = const(-22)
_MP_ETIMEDOUT = const(-110)

# MicroPython stream ioctl requests, same as py/stream.h
_MP_STREAM_FLUSH = const(1)
_MP_STREAM_POLL = const(3)

# MicroPython ioctl poll values, same as py/stream.h
_MP_STREAM_POLL_WR = const(0x04)
_MP_STREAM_POLL_RD = const(0x01)
_MP_STREAM_POLL_HUP = const(0x10)


class CDCInterface(io.IOBase, Interface):
    # USB CDC serial device class, designed to resemble machine.UART
    # with some additional methods.
    #
    # Relies on multiple inheritance so it can be an io.IOBase for stream
    # functions and also a Interface (actually an Interface Association
    # Descriptor holding two interfaces.)
    def __init__(self, **kwargs):
        # io.IOBase has no __init__()
        Interface.__init__(self)

        # Callbacks for particular control changes initiated by the host
        self.break_cb = None  # Host sent a "break" condition
        self.line_state_cb = None
        self.line_coding_cb = None

        self._line_state = 0  # DTR & RTS
        # Set a default line coding of 115200/8N1
        self._line_coding = bytearray(b"\x00\xc2\x01\x00\x00\x00\x08")

        self._wb = ()  # Optional write Buffer (IN endpoint), set by CDC.init()
        self._rb = ()  # Optional read Buffer (OUT endpoint), set by CDC.init()
        self._timeout = 1000  # set from CDC.init() as well

        # one control interface endpoint, two data interface endpoints
        self.ep_c_in = self.ep_d_in = self.ep_d_out = None

        self._c_itf = None  # Number of control interface, data interface is one more

        self.init(**kwargs)

    def init(
        self, baudrate=9600, bits=8, parity="N", stop=1, timeout=None, txbuf=256, rxbuf=256, flow=0
    ):
        # Configure the CDC serial port. Note that many of these settings like
        # baudrate, bits, parity, stop don't change the USB-CDC device behavior
        # at all, only the "line coding" as communicated from/to the USB host.

        # Store initial line coding parameters in the USB CDC binary format
        # (there is nothing implemented to further change these from Python
        # code, the USB host sets them.)
        struct.pack_into(
            "<LBBB",
            self._line_coding,
            0,
            baudrate,
            _STOP_BITS_REPR.index(str(stop)),
            _PARITY_BITS_REPR.index(parity),
            bits,
        )

        if flow != 0:
            raise NotImplementedError  # UART flow control currently not supported

        if not (txbuf and rxbuf >= _BULK_EP_LEN):
            raise ValueError  # Buffer sizes are required, rxbuf must be at least one EP

        self._timeout = timeout
        self._wb = Buffer(txbuf)
        self._rb = Buffer(rxbuf)

    ###
    ### Line State & Line Coding State property getters
    ###

    @property
    def rts(self):
        return bool(self._line_state & _LINE_STATE_RTS)

    @property
    def dtr(self):
        return bool(self._line_state & _LINE_STATE_DTR)

    # Line Coding Representation
    # Byte 0-3   Byte 4      Byte 5       Byte 6
    # dwDTERate  bCharFormat bParityType  bDataBits

    @property
    def baudrate(self):
        return struct.unpack("<LBBB", self._line_coding)[0]

    @property
    def stop_bits(self):
        return _STOP_BITS_REPR[self._line_coding[4]]

    @property
    def parity(self):
        return _PARITY_BITS_REPR[self._line_coding[5]]

    @property
    def data_bits(self):
        return self._line_coding[6]

    def __repr__(self):
        return f"{self.baudrate}/{self.data_bits}{self.parity}{self.stop_bits} rts={self.rts} dtr={self.dtr}"

    ###
    ### Set callbacks for operations initiated by the host
    ###

    def set_break_cb(self, cb):
        self.break_cb = cb

    def set_line_state_cb(self, cb):
        self.line_state_cb = cb

    def set_line_coding_cb(self, cb):
        self.line_coding_cb = cb

    ###
    ### USB Interface Implementation
    ###

    def desc_cfg(self, desc, itf_num, ep_num, strs):
        # CDC needs a Interface Association Descriptor (IAD) wrapping two interfaces: Control & Data interfaces
        desc.interface_assoc(itf_num, 2, _INTERFACE_CLASS_CDC, _INTERFACE_SUBCLASS_CDC)

        # Now add the Control interface descriptor
        self._c_itf = itf_num
        desc.interface(itf_num, _CDC_CONTROL_EP_NUM, _INTERFACE_CLASS_CDC, _INTERFACE_SUBCLASS_CDC)

        # Append the CDC class-specific interface descriptor
        # see CDC120-track, p20
        desc.pack(
            "<BBBH",
            5,  # bFunctionLength
            _CS_DESC_TYPE,  # bDescriptorType
            _CDC_FUNC_DESC_HEADER,  # bDescriptorSubtype
            _CDC_VERSION,  # cdc version
        )

        # CDC-PSTN table3 "Call Management"
        # set to No
        desc.pack(
            "<BBBBB",
            5,  # bFunctionLength
            _CS_DESC_TYPE,  # bDescriptorType
            _CDC_FUNC_DESC_CALL_MANAGEMENT,  # bDescriptorSubtype
            0,  # bmCapabilities - XXX no call managment so far
            itf_num + 1,  # bDataInterface - interface 1
        )

        # CDC-PSTN table4 "Abstract Control"
        # set to support line_coding and send_break
        desc.pack(
            "<BBBB",
            4,  # bFunctionLength
            _CS_DESC_TYPE,  # bDescriptorType
            _CDC_FUNC_DESC_ABSTRACT_CONTROL,  # bDescriptorSubtype
            0x6,  # bmCapabilities D1, D2
        )

        # CDC-PSTN "Union"
        # set control interface / data interface number
        desc.pack(
            "<BBBBB",
            5,  # bFunctionLength
            _CS_DESC_TYPE,  # bDescriptorType
            _CDC_FUNC_DESC_UNION,  # bDescriptorSubtype
            itf_num,  # bControlInterface
            itf_num + 1,  # bSubordinateInterface0 (data class itf number)
        )

        # Single control IN endpoint (currently unused in this implementation)
        self.ep_c_in = ep_num | _EP_IN_FLAG
        desc.endpoint(self.ep_c_in, "interrupt", 8, 16)

        # Now add the data interface
        desc.interface(
            itf_num + 1,
            _CDC_DATA_EP_NUM,
            _CDC_ITF_DATA_CLASS,
            _CDC_ITF_DATA_SUBCLASS,
            _CDC_ITF_DATA_PROT,
        )

        # Two data endpoints, bulk OUT and IN
        self.ep_d_out = ep_num + 1
        self.ep_d_in = (ep_num + 1) | _EP_IN_FLAG
        desc.endpoint(self.ep_d_out, "bulk", _BULK_EP_LEN, 0)
        desc.endpoint(self.ep_d_in, "bulk", _BULK_EP_LEN, 0)

    def num_itfs(self):
        return 2

    def num_eps(self):
        return 2  # total after masking out _EP_IN_FLAG

    def on_open(self):
        super().on_open()
        # kick off any transfers that may have queued while the device was not open
        self._rd_xfer()
        self._wr_xfer()

    def on_interface_control_xfer(self, stage, request):
        # Handle class-specific interface control transfers
        bmRequestType, bRequest, wValue, wIndex, wLength = struct.unpack("BBHHH", request)
        recipient, req_type, req_dir = split_bmRequestType(bmRequestType)

        if wIndex != self._c_itf:
            return False  # Only for the control interface (may be redundant check?)

        if req_type != _REQ_TYPE_CLASS:
            return False  # Unsupported request type

        if stage == _STAGE_SETUP:
            if bRequest in (_SET_LINE_CODING_REQ, _GET_LINE_CODING_REQ):
                return self._line_coding  # Buffer to read or write

            # Continue on other supported requests, stall otherwise
            return bRequest in (_SET_CONTROL_LINE_STATE, _SEND_BREAK_REQ)

        if stage == _STAGE_ACK:
            if bRequest == _SET_LINE_CODING_REQ:
                if self.line_coding_cb:
                    self.line_coding_cb(self._line_coding)
            elif bRequest == _SET_CONTROL_LINE_STATE:
                self._line_state = wValue
                if self.line_state_cb:
                    self.line_state_cb(wValue)
            elif bRequest == _SEND_BREAK_REQ:
                if self.break_cb:
                    self.break_cb(wValue)

        return True  # allow DATA/ACK stages to complete normally

    def _wr_xfer(self):
        # Submit a new data IN transfer from the _wb buffer, if needed
        if self.is_open() and not self.xfer_pending(self.ep_d_in) and self._wb.readable():
            self.submit_xfer(self.ep_d_in, self._wb.pend_read(), self._wr_cb)

    def _wr_cb(self, ep, res, num_bytes):
        # Whenever a data IN transfer ends
        if res == 0:
            self._wb.finish_read(num_bytes)
        self._wr_xfer()

    def _rd_xfer(self):
        # Keep an active data OUT transfer to read data from the host,
        # whenever the receive buffer has room for new data
        if (
            self.is_open()
            and not self.xfer_pending(self.ep_d_out)
            and self._rb.writable() >= _BULK_EP_LEN
        ):
            # Can only submit up to the endpoint length per transaction, otherwise we won't
            # get any transfer callback until the full transaction completes.
            self.submit_xfer(self.ep_d_out, self._rb.pend_write(_BULK_EP_LEN), self._rd_cb)

    def _rd_cb(self, ep, res, num_bytes):
        # Whenever a data OUT transfer ends
        if res == 0:
            self._rb.finish_write(num_bytes)
        self._rd_xfer()

    ###
    ### io.IOBase stream implementation
    ###

    def write(self, buf):
        # use a memoryview to track how much of 'buf' we've written so far
        # (unfortunately, this means a 1 block allocation for each write, but it's otherwise allocation free.)
        start = time.ticks_ms()
        mv = memoryview(buf)
        while True:
            # Keep pushing buf into _wb into it's all gone
            nbytes = self._wb.write(mv)
            self._wr_xfer()  # make sure a transfer is running from _wb

            if nbytes == len(mv):
                return len(buf)  # Success

            mv = mv[nbytes:]

            # check for timeout
            if time.ticks_diff(time.ticks_ms(), start) >= self._timeout:
                return len(buf) - len(mv)

            machine.idle()

    def read(self, size):
        start = time.ticks_ms()

        # Allocate a suitable buffer to read into
        if size >= 0:
            b = bytearray(size)
        else:
            # for size == -1, return however many bytes are ready
            b = bytearray(self._rb.readable())

        n = self._readinto(b, start)
        if not n:
            return None
        if n < len(b):
            return b[:n]
        return b

    def readinto(self, b):
        return self._readinto(b, time.ticks_ms())

    def _readinto(self, b, start):
        if len(b) == 0:
            return 0

        n = 0
        m = memoryview(b)
        while n < len(b):
            # copy out of the read buffer if there is anything available
            if self._rb.readable():
                n += self._rb.readinto(m if n == 0 else m[n:])
                self._rd_xfer()  # if _rd was previously full, no transfer will be running
                if n == len(b):
                    break  # Done, exit before we call machine.idle()

            if time.ticks_diff(time.ticks_ms(), start) >= self._timeout:
                break  # Timed out

            machine.idle()

        return n or None

    def ioctl(self, req, arg):
        if req == _MP_STREAM_POLL:
            return (
                (_MP_STREAM_POLL_WR if (arg & _MP_STREAM_POLL_WR) and self._wb.writable() else 0)
                | (_MP_STREAM_POLL_RD if (arg & _MP_STREAM_POLL_RD) and self._rb.readable() else 0)
                |
                # using the USB level "open" (i.e. connected to host) for !HUP, not !DTR (port is open)
                (_MP_STREAM_POLL_HUP if (arg & _MP_STREAM_POLL_HUP) and not self.is_open() else 0)
            )
        elif req == _MP_STREAM_FLUSH:
            start = time.ticks_ms()
            # Wait until write buffer contains no bytes for the lower TinyUSB layer to "read"
            while self._wb.readable():
                if not self.is_open():
                    return _MP_EINVAL
                if time.ticks_diff(time.ticks_ms(), start) > self._timeout:
                    return _MP_ETIMEDOUT
                machine.idle()
            return 0

        return _MP_EINVAL

    def flush(self):
        # a C implementation of this exists in stream.c, but it's not in io.IOBase
        # and can't immediately be called from here (AFAIK)
        r = self.ioctl(_MP_STREAM_FLUSH, 0)
        if r:
            raise OSError(r)



# This makes unittest return an error code, so is not named "test_xxx.py".

import unittest


def broken_func():
    raise ValueError("uh oh!")


def test_func():
    broken_func()


if __name__ == "__main__":
    unittest.main()



"""
Reset script for the cc3200 boards
This is needed to force the board to reboot
with the default WLAN AP settings
"""

from machine import WDT
import time
import os

mch = os.uname().machine
if not "LaunchPad" in mch and not "WiPy" in mch:
    raise Exception("Board not supported!")

wdt = WDT(timeout=1000)
print(wdt)
time.sleep_ms(900)



# Test the esp32's esp32.idf_heap_info to return sane data
try:
    import esp32
except ImportError:
    print("SKIP")
    raise SystemExit


# region tuple is: (size, free, largest free, min free)
# we check that each region's size is > 0 and that the free amounts are smaller than the size
def chk_heap(kind, regions):
    chk = [True, True, True, True]
    for r in regions:
        chk = [
            chk[0] and r[0] > 0,
            chk[1] and r[1] <= r[0],
            chk[2] and r[2] <= r[0],
            chk[3] and r[3] <= r[0],
        ]
    print(kind, chk)


# try getting heap regions
regions = esp32.idf_heap_info(esp32.HEAP_DATA)
print("HEAP_DATA >2:", len(regions) > 2)
chk_heap("HEAP_DATA", regions)

# try getting code regions
regions = esp32.idf_heap_info(esp32.HEAP_EXEC)
print("HEAP_EXEC >2:", len(regions) > 2)
chk_heap("HEAP_EXEC", regions)

# try invalid param
print(esp32.idf_heap_info(-1))



# This file is part of the MicroPython project, http://micropython.org/
#
# The MIT License (MIT)
#
# Copyright (c) 2016 Rami Ali
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.

""" gen-cpydiff generates documentation which outlines operations that differ between MicroPython
    and CPython. This script is called by the docs Makefile for html and Latex and may be run
    manually using the command make gen-cpydiff. """

import os
import subprocess
import time
import re
from collections import namedtuple

# MicroPython supports syntax of CPython 3.4 with some features from 3.5, and
# such version should be used to test for differences. If your default python3
# executable is of lower version, you can point MICROPY_CPYTHON3 environment var
# to the correct executable.
if os.name == "nt":
    CPYTHON3 = os.getenv("MICROPY_CPYTHON3", "python3.exe")
    MICROPYTHON = os.getenv(
        "MICROPY_MICROPYTHON", "../ports/windows/build-standard/micropython.exe"
    )
else:
    CPYTHON3 = os.getenv("MICROPY_CPYTHON3", "python3")
    MICROPYTHON = os.getenv("MICROPY_MICROPYTHON", "../ports/unix/build-standard/micropython")

TESTPATH = "../tests/cpydiff"
DOCPATH = "../docs/genrst"
SRCDIR = "../docs/differences"
INDEXTEMPLATE = os.path.join(SRCDIR, "index_template.txt")
INDEX = "index.rst"

HEADER = ".. This document was generated by tools/gen-cpydiff.py\n\n"
CLASSMAP = {"Core": "Core language", "Types": "Builtin types"}
INDEXPRIORITY = ["syntax", "core_language", "builtin_types", "modules"]
RSTCHARS = ["=", "-", "~", "`", ":"]
SPLIT = '"""\n|categories: |description: |cause: |workaround: '
TAB = "    "

Output = namedtuple(
    "output",
    [
        "name",
        "class_",
        "desc",
        "cause",
        "workaround",
        "code",
        "output_cpy",
        "output_upy",
        "status",
    ],
)


def readfiles():
    """Reads test files"""
    tests = list(filter(lambda x: x.endswith(".py"), os.listdir(TESTPATH)))
    tests.sort()
    files = []

    for test in tests:
        test_fullpath = os.path.join(TESTPATH, test)
        text = open(test_fullpath, "r").read()

        try:
            class_, desc, cause, workaround, code = [
                x.rstrip() for x in list(filter(None, re.split(SPLIT, text)))
            ]

            # remove black `fmt: on/off/skip` comments
            code = "".join(
                # skip comments are inline, so we replace just the comment
                re.sub(r"\s*# fmt: skip", "", x)
                for x in code.splitlines(keepends=True)
                # on/off comments are on their own line, so we omit the entire line
                if not re.match(r"\s*# fmt: (on|off)\s*", x)
            )

            output = Output(test, class_, desc, cause, workaround, code, "", "", "")
            files.append(output)
        except IndexError:
            print("Incorrect format in file " + test_fullpath)

    return files


def run_tests(tests):
    """executes all tests"""
    results = []
    for test in tests:
        test_fullpath = os.path.join(TESTPATH, test.name)
        with open(test_fullpath, "rb") as f:
            input_py = f.read()

        process = subprocess.Popen(
            CPYTHON3,
            shell=True,
            stdout=subprocess.PIPE,
            stdin=subprocess.PIPE,
            stderr=subprocess.PIPE,
        )
        output_cpy = [com.decode("utf8") for com in process.communicate(input_py)]

        process = subprocess.Popen(
            MICROPYTHON,
            shell=True,
            stdout=subprocess.PIPE,
            stdin=subprocess.PIPE,
            stderr=subprocess.PIPE,
        )
        output_upy = [com.decode("utf8") for com in process.communicate(input_py)]

        if output_cpy[0] == output_upy[0] and output_cpy[1] == output_upy[1]:
            status = "Supported"
            print("Supported operation!\nFile: " + test_fullpath)
        else:
            status = "Unsupported"

        output = Output(
            test.name,
            test.class_,
            test.desc,
            test.cause,
            test.workaround,
            test.code,
            output_cpy,
            output_upy,
            status,
        )
        results.append(output)

    results.sort(key=lambda x: x.class_)
    return results


def indent(block, spaces):
    """indents paragraphs of text for rst formatting"""
    new_block = ""
    for line in block.split("\n"):
        new_block += spaces + line + "\n"
    return new_block


def gen_table(contents):
    """creates a table given any set of columns"""
    xlengths = []
    ylengths = []
    for column in contents:
        col_len = 0
        for entry in column:
            lines = entry.split("\n")
            for line in lines:
                col_len = max(len(line) + 2, col_len)
        xlengths.append(col_len)
    for i in range(len(contents[0])):
        ymax = 0
        for j in range(len(contents)):
            ymax = max(ymax, len(contents[j][i].split("\n")))
        ylengths.append(ymax)

    table_divider = "+" + "".join(["-" * i + "+" for i in xlengths]) + "\n"
    table = table_divider
    for i in range(len(ylengths)):
        row = [column[i] for column in contents]
        row = [entry + "\n" * (ylengths[i] - len(entry.split("\n"))) for entry in row]
        row = [entry.split("\n") for entry in row]
        for j in range(ylengths[i]):
            k = 0
            for entry in row:
                width = xlengths[k]
                table += "".join(["| {:{}}".format(entry[j], width - 1)])
                k += 1
            table += "|\n"
        table += table_divider
    return table + "\n"


def gen_rst(results):
    """creates restructured text documents to display tests"""

    # make sure the destination directory exists
    if not os.path.isdir(DOCPATH):
        os.mkdir(DOCPATH)

    toctree = []
    class_ = []
    for output in results:
        section = output.class_.split(",")
        for i in range(len(section)):
            section[i] = section[i].rstrip()
            if section[i] in CLASSMAP:
                section[i] = CLASSMAP[section[i]]
            if i >= len(class_) or section[i] != class_[i]:
                if i == 0:
                    filename = section[i].replace(" ", "_").lower()
                    rst = open(os.path.join(DOCPATH, filename + ".rst"), "w")
                    rst.write(HEADER)
                    rst.write(section[i] + "\n")
                    rst.write(RSTCHARS[0] * len(section[i]))
                    rst.write(time.strftime("\nGenerated %a %d %b %Y %X UTC\n\n", time.gmtime()))
                    # If a file docs/differences/<filename>_preamble.txt exists
                    # then its output is inserted after the top-level heading,
                    # but before any of the generated sections.
                    preamble_path = os.path.join(SRCDIR, filename + "_preamble.txt")
                    if os.path.exists(preamble_path):
                        with open(preamble_path, "r") as f:
                            rst.write(f.read())
                        rst.write("\n")
                    toctree.append(filename)
                else:
                    rst.write(section[i] + "\n")
                    rst.write(RSTCHARS[min(i, len(RSTCHARS) - 1)] * len(section[i]))
                    rst.write("\n\n")
        class_ = section
        rst.write(".. _cpydiff_%s:\n\n" % os.path.splitext(output.name)[0])
        rst.write(output.desc + "\n")
        rst.write("~" * len(output.desc) + "\n\n")
        if output.cause != "Unknown":
            rst.write("**Cause:** " + output.cause + "\n\n")
        if output.workaround != "Unknown":
            rst.write("**Workaround:** " + output.workaround + "\n\n")

        rst.write("Sample code::\n\n" + indent(output.code, TAB) + "\n")
        output_cpy = indent("".join(output.output_cpy[0:2]), TAB).rstrip()
        output_cpy = ("::\n\n" if output_cpy != "" else "") + output_cpy
        output_upy = indent("".join(output.output_upy[0:2]), TAB).rstrip()
        output_upy = ("::\n\n" if output_upy != "" else "") + output_upy
        table = gen_table([["CPy output:", output_cpy], ["uPy output:", output_upy]])
        rst.write(table)

    template = open(INDEXTEMPLATE, "r")
    index = open(os.path.join(DOCPATH, INDEX), "w")
    index.write(HEADER)
    index.write(template.read())
    for section in INDEXPRIORITY:
        if section in toctree:
            index.write(indent(section + ".rst", TAB))
            toctree.remove(section)
    for section in toctree:
        index.write(indent(section + ".rst", TAB))


def main():
    """Main function"""

    # set search path so that test scripts find the test modules (and no other ones)
    os.environ["PYTHONPATH"] = TESTPATH
    os.environ["MICROPYPATH"] = TESTPATH

    files = readfiles()
    results = run_tests(files)
    gen_rst(results)


main()



from time import sleep_ms
from ssd1306 import SSD1306_I2C
import network


class OLED(SSD1306_I2C):
    def __init__(self, i2c, rstpin):
        # Initialize the OLED display
        if rstpin is not None:
            rstpin.value(0)
            sleep_ms(50)
            rstpin.value(1)  # must be held high after initialization
        super().__init__(128, 32, i2c)

    def test(self):
        self.fill(0)
        self.fill_rect(0, 0, 32, 32, 1)
        self.fill_rect(2, 2, 28, 28, 0)
        self.vline(9, 8, 22, 1)
        self.vline(16, 2, 22, 1)
        self.vline(23, 8, 22, 1)
        self.fill_rect(26, 24, 2, 4, 1)
        self.text("MicroPython", 40, 0, 1)
        self.text("SSD1306", 40, 12, 1)
        self.text("OLED 128x32", 40, 24, 1)
        self.show()

    def display_wifi(self):
        self.fill(0)
        self.text("Scan...", 0, 0, 1)
        self.show()

        sta_if = network.WLAN(network.STA_IF)
        sta_if.active(True)
        _wifi = sta_if.scan()

        self.fill(0)
        self.text(str(len(_wifi)) + " Networks", 0, 0, 1)
        self.text(str(_wifi[0][3]) + " " + (_wifi[0][0]).decode("utf-8"), 0, 12, 1)
        self.text(str(_wifi[1][3]) + " " + (_wifi[1][0]).decode("utf-8"), 0, 24, 1)
        self.show()



# LOLIN C3 MINI MicroPython Helper Library

from micropython import const
from machine import Pin

# Pin Assignments

# SPI
SPI_MOSI = const(4)
SPI_MISO = const(3)
SPI_CLK = const(2)

# I2C
I2C_SDA = const(8)
I2C_SCL = const(10)

# LED
LED = const(7)

# BUTTON
BUTTON = const(0)

# Built-in peripherals

led = Pin(LED, Pin.OUT, value=0)
button = Pin(BUTTON, Pin.IN, Pin.PULL_UP)



"""
Pure Python, iperf3-compatible network performance test tool.

MIT license; Copyright (c) 2018-2019 Damien P. George

Supported modes: server & client, TCP & UDP, normal & reverse

Usage:
    import iperf3
    iperf3.server()
    iperf3.client('192.168.1.5')
    iperf3.client('192.168.1.5', udp=True, reverse=True)
"""

import json
import select
import socket
import struct
import sys
import time

# Provide a urandom() function, supporting devices without os.urandom().
try:
    from os import urandom
except ImportError:
    from random import randint

    def urandom(n):
        return bytes(randint(0, 255) for _ in range(n))


DEBUG = False

# iperf3 cookie size, last byte is null byte
COOKIE_SIZE = 37

# iperf3 commands
TEST_START = 1
TEST_RUNNING = 2
TEST_END = 4
PARAM_EXCHANGE = 9
CREATE_STREAMS = 10
EXCHANGE_RESULTS = 13
DISPLAY_RESULTS = 14
IPERF_DONE = 16

if DEBUG:
    cmd_string = {
        TEST_START: "TEST_START",
        TEST_RUNNING: "TEST_RUNNING",
        TEST_END: "TEST_END",
        PARAM_EXCHANGE: "PARAM_EXCHANGE",
        CREATE_STREAMS: "CREATE_STREAMS",
        EXCHANGE_RESULTS: "EXCHANGE_RESULTS",
        DISPLAY_RESULTS: "DISPLAY_RESULTS",
        IPERF_DONE: "IPERF_DONE",
    }


def fmt_size(val, div):
    for mult in ("", "K", "M", "G"):
        if val < 10:
            return "% 5.2f %s" % (val, mult)
        elif val < 100:
            return "% 5.1f %s" % (val, mult)
        elif mult == "G" or val < 1000:
            return "% 5.0f %s" % (val, mult)
        else:
            val /= div


class Stats:
    def __init__(self, param):
        self.pacing_timer_us = param["pacing_timer"] * 1000
        self.udp = param.get("udp", False)
        self.reverse = param.get("reverse", False)
        self.running = False

    def start(self):
        self.running = True
        self.t0 = self.t1 = ticks_us()
        self.nb0 = self.nb1 = 0  # num bytes
        self.np0 = self.np1 = 0  # num packets
        self.nm0 = self.nm1 = 0  # num lost packets
        if self.udp:
            if self.reverse:
                extra = "         Jitter    Lost/Total Datagrams"
            else:
                extra = "         Total Datagrams"
        else:
            extra = ""
        print("Interval           Transfer     Bitrate" + extra)

    def max_dt_ms(self):
        if not self.running:
            return -1
        return max(0, (self.pacing_timer_us - ticks_diff(ticks_us(), self.t1)) // 1000)

    def add_bytes(self, n):
        if not self.running:
            return
        self.nb0 += n
        self.nb1 += n
        self.np0 += 1
        self.np1 += 1

    def add_lost_packets(self, n):
        self.np0 += n
        self.np1 += n
        self.nm0 += n
        self.nm1 += n

    def print_line(self, ta, tb, nb, np, nm, extra=""):
        dt = tb - ta
        print(
            " %5.2f-%-5.2f  sec %sBytes %sbits/sec"
            % (ta, tb, fmt_size(nb, 1024), fmt_size(nb * 8 / dt, 1000)),
            end="",
        )
        if self.udp:
            if self.reverse:
                print(
                    " %6.3f ms  %u/%u (%.1f%%)" % (0, nm, np, 100 * nm / (max(1, np + nm))), end=""
                )
            else:
                print("  %u" % np, end="")
        print(extra)

    def update(self, final=False):
        if not self.running:
            return
        t2 = ticks_us()
        dt = ticks_diff(t2, self.t1)
        if final or dt > self.pacing_timer_us:
            ta = ticks_diff(self.t1, self.t0) * 1e-6
            tb = ticks_diff(t2, self.t0) * 1e-6
            self.print_line(ta, tb, self.nb1, self.np1, self.nm1)
            self.t1 = t2
            self.nb1 = 0
            self.np1 = 0
            self.nm1 = 0

    def stop(self):
        self.update(True)
        self.running = False
        self.t3 = ticks_us()
        dt = ticks_diff(self.t3, self.t0)
        print("- " * 30)
        self.print_line(0, dt * 1e-6, self.nb0, self.np0, self.nm0, "  sender")

    def report_receiver(self, stats):
        st = stats["streams"][0]

        # iperf servers pre 3.2 do not transmit start or end time,
        # so use local as fallback if not available.
        dt = ticks_diff(self.t3, self.t0)

        self.print_line(
            st.get("start_time", 0.0),
            st.get("end_time", dt * 1e-6),
            st["bytes"],
            st["packets"],
            st["errors"],
            "  receiver",
        )


def recvn(s, n):
    data = b""
    while len(data) < n:
        data += s.recv(n - len(data))
    return data


def recvinto(s, buf):
    if hasattr(s, "readinto"):
        return s.readinto(buf)
    else:
        return s.recv_into(buf)


def recvninto(s, buf):
    if hasattr(s, "readinto"):
        n = s.readinto(buf)
        assert n == len(buf)
    else:
        mv = memoryview(buf)
        off = 0
        while off < len(buf):
            off += s.recv_into(mv[off:])


def make_cookie():
    cookie_chars = b"abcdefghijklmnopqrstuvwxyz234567"
    cookie = bytearray(COOKIE_SIZE)
    for i, x in enumerate(urandom(COOKIE_SIZE - 1)):
        cookie[i] = cookie_chars[x & 31]
    return cookie


def server_once():
    # Listen for a connection
    ai = socket.getaddrinfo("0.0.0.0", 5201)
    ai = ai[0]
    print("Server listening on", ai[-1])
    s_listen = socket.socket(ai[0], socket.SOCK_STREAM)
    s_listen.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
    s_listen.bind(ai[-1])
    s_listen.listen(1)
    s_ctrl, addr = s_listen.accept()

    # Read client's cookie
    cookie = recvn(s_ctrl, COOKIE_SIZE)
    if DEBUG:
        print(cookie)

    # Ask for parameters
    s_ctrl.sendall(bytes([PARAM_EXCHANGE]))

    # Get parameters
    n = struct.unpack(">I", recvn(s_ctrl, 4))[0]
    param = recvn(s_ctrl, n)
    param = json.loads(str(param, "ascii"))
    if DEBUG:
        print(param)
    reverse = param.get("reverse", False)

    # Ask to create streams
    s_ctrl.sendall(bytes([CREATE_STREAMS]))

    if param.get("tcp", False):
        # Accept stream
        s_data, addr = s_listen.accept()
        print("Accepted connection:", addr)
        recvn(s_data, COOKIE_SIZE)
    elif param.get("udp", False):
        # Close TCP connection and open UDP "connection"
        s_listen.close()
        s_data = socket.socket(ai[0], socket.SOCK_DGRAM)
        s_data.bind(ai[-1])
        data, addr = s_data.recvfrom(4)
        s_data.sendto(b"\x12\x34\x56\x78", addr)
    else:
        assert False

    # Start test
    s_ctrl.sendall(bytes([TEST_START]))

    # Run test
    s_ctrl.sendall(bytes([TEST_RUNNING]))

    # Read data, and wait for client to send TEST_END
    poll = select.poll()
    poll.register(s_ctrl, select.POLLIN)
    if reverse:
        poll.register(s_data, select.POLLOUT)
    else:
        poll.register(s_data, select.POLLIN)
    stats = Stats(param)
    stats.start()
    running = True
    data_buf = bytearray(urandom(param["len"]))
    while running:
        for pollable in poll.poll(stats.max_dt_ms()):
            if pollable_is_sock(pollable, s_ctrl):
                cmd = recvn(s_ctrl, 1)[0]
                if DEBUG:
                    print(cmd_string.get(cmd, "UNKNOWN_COMMAND"))
                if cmd == TEST_END:
                    running = False
            elif pollable_is_sock(pollable, s_data):
                if reverse:
                    n = s_data.send(data_buf)
                    stats.add_bytes(n)
                else:
                    recvninto(s_data, data_buf)
                    stats.add_bytes(len(data_buf))
        stats.update()

    # Need to continue writing so other side doesn't get blocked waiting for data
    if reverse:
        while True:
            for pollable in poll.poll(0):
                if pollable_is_sock(pollable, s_data):
                    s_data.send(data_buf)
                    break
            else:
                break

    stats.stop()

    # Ask to exchange results
    s_ctrl.sendall(bytes([EXCHANGE_RESULTS]))

    # Get client results
    n = struct.unpack(">I", recvn(s_ctrl, 4))[0]
    results = recvn(s_ctrl, n)
    results = json.loads(str(results, "ascii"))
    if DEBUG:
        print(results)

    # Send our results
    results = {
        "cpu_util_total": 1,
        "cpu_util_user": 0.5,
        "cpu_util_system": 0.5,
        "sender_has_retransmits": 1,
        "congestion_used": "cubic",
        "streams": [
            {
                "id": 1,
                "bytes": stats.nb0,
                "retransmits": 0,
                "jitter": 0,
                "errors": 0,
                "packets": stats.np0,
                "start_time": 0,
                "end_time": ticks_diff(stats.t3, stats.t0) * 1e-6,
            }
        ],
    }
    results = json.dumps(results)
    s_ctrl.sendall(struct.pack(">I", len(results)))
    s_ctrl.sendall(bytes(results, "ascii"))

    # Ask to display results
    s_ctrl.sendall(bytes([DISPLAY_RESULTS]))

    # Wait for client to send IPERF_DONE
    cmd = recvn(s_ctrl, 1)[0]
    assert cmd == IPERF_DONE

    # Close all sockets
    s_data.close()
    s_ctrl.close()
    s_listen.close()


def server():
    while True:
        server_once()


def client(host, udp=False, reverse=False, bandwidth=10 * 1024 * 1024):
    print("CLIENT MODE:", "UDP" if udp else "TCP", "receiving" if reverse else "sending")

    param = {
        "client_version": "3.6",
        "omit": 0,
        "parallel": 1,
        "pacing_timer": 1000,
        "time": 10,
    }

    if udp:
        param["udp"] = True
        param["len"] = 1500 - 42
        param["bandwidth"] = bandwidth  # this should be should be intended bits per second
        udp_interval = 1000000 * 8 * param["len"] // param["bandwidth"]
    else:
        param["tcp"] = True
        param["len"] = 3000

    if reverse:
        param["reverse"] = True

    # Connect to server
    ai = socket.getaddrinfo(host, 5201)[0]
    print("Connecting to", ai[-1])
    s_ctrl = socket.socket(ai[0], socket.SOCK_STREAM)
    s_ctrl.connect(ai[-1])

    # Send our cookie
    cookie = make_cookie()
    if DEBUG:
        print(cookie)
    s_ctrl.sendall(cookie)

    # Object to gather statistics about the run
    stats = Stats(param)

    # Run the main loop, waiting for incoming commands and dat
    ticks_us_end = param["time"] * 1000000
    poll = select.poll()
    poll.register(s_ctrl, select.POLLIN)
    buf = None
    s_data = None
    start = None
    udp_packet_id = 0
    udp_last_send = None
    while True:
        for pollable in poll.poll(stats.max_dt_ms()):
            if pollable_is_sock(pollable, s_data):
                # Data socket is writable/readable
                t = ticks_us()
                if ticks_diff(t, start) > ticks_us_end:
                    if reverse:
                        # Continue to drain any incoming data
                        recvinto(s_data, buf)
                    if stats.running:
                        # End of run
                        s_ctrl.sendall(bytes([TEST_END]))
                        stats.stop()
                else:
                    # Send/receiver data
                    if udp:
                        if reverse:
                            recvninto(s_data, buf)
                            udp_in_sec, udp_in_usec, udp_in_id = struct.unpack_from(">III", buf, 0)
                            # print(udp_in_sec, udp_in_usec, udp_in_id)
                            if udp_in_id != udp_packet_id + 1:
                                stats.add_lost_packets(udp_in_id - (udp_packet_id + 1))
                            udp_packet_id = udp_in_id
                            stats.add_bytes(len(buf))
                        else:
                            # print('UDP send', udp_last_send, t, udp_interval)
                            if t - udp_last_send > udp_interval:
                                udp_last_send += udp_interval
                                udp_packet_id += 1
                                struct.pack_into(
                                    ">III", buf, 0, t // 1000000, t % 1000000, udp_packet_id
                                )
                                n = s_data.sendto(buf, ai[-1])
                                stats.add_bytes(n)
                    else:
                        if reverse:
                            recvninto(s_data, buf)
                            n = len(buf)
                        else:
                            # print('TCP send', len(buf))
                            n = s_data.send(buf)
                        stats.add_bytes(n)

            elif pollable_is_sock(pollable, s_ctrl):
                # Receive command
                cmd = recvn(s_ctrl, 1)[0]
                if DEBUG:
                    print(cmd_string.get(cmd, "UNKNOWN_COMMAND"))
                if cmd == TEST_START:
                    if reverse:
                        # Start receiving data now, because data socket is open
                        poll.register(s_data, select.POLLIN)
                        start = ticks_us()
                        stats.start()
                elif cmd == TEST_RUNNING:
                    if not reverse:
                        # Start sending data now
                        poll.register(s_data, select.POLLOUT)
                        start = ticks_us()
                        if udp:
                            udp_last_send = start - udp_interval
                        stats.start()
                elif cmd == PARAM_EXCHANGE:
                    param_j = json.dumps(param)
                    s_ctrl.sendall(struct.pack(">I", len(param_j)))
                    s_ctrl.sendall(bytes(param_j, "ascii"))
                elif cmd == CREATE_STREAMS:
                    if udp:
                        s_data = socket.socket(ai[0], socket.SOCK_DGRAM)
                        s_data.sendto(struct.pack("<I", 123456789), ai[-1])
                        recvn(s_data, 4)  # get dummy response from server (=987654321)
                    else:
                        s_data = socket.socket(ai[0], socket.SOCK_STREAM)
                        s_data.connect(ai[-1])
                        s_data.sendall(cookie)
                    buf = bytearray(urandom(param["len"]))
                elif cmd == EXCHANGE_RESULTS:
                    # Close data socket now that server knows we are finished, to prevent it flooding us
                    poll.unregister(s_data)
                    s_data.close()
                    s_data = None

                    results = {
                        "cpu_util_total": 1,
                        "cpu_util_user": 0.5,
                        "cpu_util_system": 0.5,
                        "sender_has_retransmits": 1,
                        "congestion_used": "cubic",
                        "streams": [
                            {
                                "id": 1,
                                "bytes": stats.nb0,
                                "retransmits": 0,
                                "jitter": 0,
                                "errors": stats.nm0,
                                "packets": stats.np0,
                                "start_time": 0,
                                "end_time": ticks_diff(stats.t3, stats.t0) * 1e-6,
                            }
                        ],
                    }
                    results = json.dumps(results)
                    s_ctrl.sendall(struct.pack(">I", len(results)))
                    s_ctrl.sendall(bytes(results, "ascii"))

                    n = struct.unpack(">I", recvn(s_ctrl, 4))[0]
                    results = recvn(s_ctrl, n)
                    results = json.loads(str(results, "ascii"))
                    stats.report_receiver(results)

                elif cmd == DISPLAY_RESULTS:
                    s_ctrl.sendall(bytes([IPERF_DONE]))
                    s_ctrl.close()
                    time.sleep(1)  # delay so server is ready for any subsequent client connections
                    return

        stats.update()


def main():
    opt_mode = None
    opt_udp = False
    opt_reverse = False

    sys.argv.pop(0)
    while sys.argv:
        opt = sys.argv.pop(0)
        if opt == "-R":
            opt_reverse = True
        elif opt == "-u":
            opt_udp = True
        elif opt == "-s":
            opt_mode = opt
        elif opt == "-c":
            opt_mode = opt
            opt_host = sys.argv.pop(0)
        else:
            print("unknown option:", opt)
            raise SystemExit(1)

    if opt_mode == "-s":
        server()
    else:
        client(opt_host, opt_udp, opt_reverse)


if sys.platform == "linux":

    def pollable_is_sock(pollable, sock):
        return sock is not None and pollable[0] == sock.fileno()

    def ticks_us():
        return int(time.time() * 1e6)

    def ticks_diff(a, b):
        return a - b

    if __name__ == "__main__":
        main()
else:

    def pollable_is_sock(pollable, sock):
        return pollable[0] == sock

    from time import ticks_us, ticks_diff



# Check that micropython.RingIO works correctly with asyncio.Stream.

import micropython

try:
    import asyncio

    asyncio.StreamWriter
    micropython.RingIO
except (AttributeError, ImportError):
    print("SKIP")
    raise SystemExit

rb = micropython.RingIO(16)
rba = asyncio.StreamWriter(rb)

data = b"ABC123" * 20
print("w", len(data))


async def data_writer():
    global data
    rba.write(data)
    await rba.drain()


async def main():
    task = asyncio.create_task(data_writer())
    await asyncio.sleep_ms(10)
    read = await rba.readexactly(len(data))
    print(read)
    print("r", len(read))
    print(read == data)


asyncio.run(main())



# test importing of invalid .mpy files

try:
    import sys, io, vfs

    sys.implementation._mpy
    io.IOBase
except (ImportError, AttributeError):
    print("SKIP")
    raise SystemExit


class UserFile(io.IOBase):
    def __init__(self, data):
        self.data = memoryview(data)
        self.pos = 0

    def readinto(self, buf):
        n = min(len(buf), len(self.data) - self.pos)
        buf[:n] = self.data[self.pos : self.pos + n]
        self.pos += n
        return n

    def ioctl(self, req, arg):
        return 0


class UserFS:
    def __init__(self, files):
        self.files = files

    def mount(self, readonly, mksfs):
        pass

    def umount(self):
        pass

    def stat(self, path):
        if path in self.files:
            return (32768, 0, 0, 0, 0, 0, 0, 0, 0, 0)
        raise OSError

    def open(self, path, mode):
        return UserFile(self.files[path])


# these are the test .mpy files
user_files = {
    "/mod0.mpy": b"",  # empty file
    "/mod1.mpy": b"M",  # too short header
    "/mod2.mpy": b"M\x00\x00\x00",  # bad version
}

# create and mount a user filesystem
vfs.mount(UserFS(user_files), "/userfs")
sys.path.append("/userfs")

# import .mpy files from the user filesystem
for i in range(len(user_files)):
    mod = "mod%u" % i
    try:
        __import__(mod)
    except ValueError as er:
        print(mod, "ValueError", er)

# unmount and undo path addition
vfs.umount("/userfs")
sys.path.pop()



import json

inp = ["foo", {"bar": ("baz", None, 1, 2)}]
print(inp)

s = json.dumps(inp)
print(s)

outp = json.loads(s)
print(outp)

# Doesn't work because JSON doesn't have tuples
# assert inp == outp



try:
    import select
    import ssl
    import io
except ImportError:
    print("SKIP")
    raise SystemExit

from micropython import const

_MP_STREAM_POLL_RD = const(0x0001)
_MP_STREAM_POLL_WR = const(0x0004)
_MP_STREAM_POLL_NVAL = const(0x0020)
_MP_STREAM_POLL = const(3)
_MP_STREAM_CLOSE = const(4)


# This self-signed key/cert pair is randomly generated and to be used for
# testing/demonstration only.  You should always generate your own key/cert.
# They has been generated using the following commands:
# $ openssl req -x509 -newkey rsa:1024 -keyout rsa_key.pem -out rsa_cert.pem -days 3650 -nodes -subj '/CN=micropython.local/O=MicroPython/C=AU'
# $ openssl pkey -in rsa_key.pem -out rsa_key.der -outform DER
# $ openssl x509 -in rsa_cert.pem -out rsa_cert.der -outform DER
key = bytes.fromhex(
    "3082025d02010002818100eca28b2f8230237ae45e7a77ef495c05a786f423cc65caf6bc"
    "1813d50eacf9d2d011a0e43a20fde947ff957075e4b3b6ded46c33f63af42597aac1c4cb"
    "bb2d1a6aad91755707d8fc560e222b38c940a480da89ee849049613bd88d1ab283423aba"
    "ba591c4fcc8ce89d19646e6b9f80de4efa7bbda68c569a5cbdb4235c3fc9990203010001"
    "0281810089d4df978b90388a534c88af252ca8b20e7377ef0616609338da196b27422fbd"
    "d03e04660b49be3bcc191dd5448632fb986d489d3795fd318c5704c879168c5cd0fa8551"
    "f7f86073b95b092ffdb4f39c867a306a02f94cf3009df7055dab1f9277dd8914268d53a2"
    "bd4de2cdf2ac90d8cd248b48868cb911781779750c344ae5024100f8c0d2cebfaccbdb1b"
    "d8bc7519c84889cc6d5dff8fb994cf1a9492881289de66d689afb942d10dc0dc0f65464e"
    "7cca7a53e451e6a8cfab0069d05065d56ba4bf024100f38757e2fc7f786e1a653c8a8b51"
    "5b06a1d85db31998090fc4d52a88b8c5557e0a7bac10995e7e76cef6bfb59b67c01cbba7"
    "edcfa7d3d623615a92459d07efa702407b149579fcf717caeb455b4229a6a2d5a3d3bd7d"
    "d4f4833fd22c0f30cad372bab98e58e736bb4fadcc74c5ac7aeb5e1816e852e9e93f0fa2"
    "5db8d7fefb118eeb024100caf0a0a6c1b02055f09d28c473b10a600e8356222853f04939"
    "c84237c97278fa1e164d9f4f8fd56780b553b12d9e5a1417b5ab91fed3a381bc6153bfbb"
    "4a9fc5024043eb4ff7e4dd6c12c6d6dc50977ee5d5f9730af4469d1f642321320fb4b969"
    "90ed841e41bedda49ff89a0c28acf132e4af5f3ac406b1548d0f135aff6bf23ef1"
)
cert = bytes.fromhex(
    "3082025a308201c3a003020102021475fd479918562f4c6cb08f63e8afbdfa3b884f8630"
    "0d06092a864886f70d01010b0500303f311a301806035504030c116d6963726f70797468"
    "6f6e2e6c6f63616c31143012060355040a0c0b4d6963726f507974686f6e310b30090603"
    "55040613024155301e170d3234303730323030353931355a170d33343036333030303539"
    "31355a303f311a301806035504030c116d6963726f707974686f6e2e6c6f63616c311430"
    "12060355040a0c0b4d6963726f507974686f6e310b300906035504061302415530819f30"
    "0d06092a864886f70d010101050003818d0030818902818100eca28b2f8230237ae45e7a"
    "77ef495c05a786f423cc65caf6bc1813d50eacf9d2d011a0e43a20fde947ff957075e4b3"
    "b6ded46c33f63af42597aac1c4cbbb2d1a6aad91755707d8fc560e222b38c940a480da89"
    "ee849049613bd88d1ab283423ababa591c4fcc8ce89d19646e6b9f80de4efa7bbda68c56"
    "9a5cbdb4235c3fc9990203010001a3533051301d0603551d0e04160414409545477a659a"
    "16da174810ba9ad192ef962089301f0603551d23041830168014409545477a659a16da17"
    "4810ba9ad192ef962089300f0603551d130101ff040530030101ff300d06092a864886f7"
    "0d01010b0500038181007182e78cecceef00f98d0ee117cd9dc2f9fc84d581e7b1d9d43b"
    "74db45e188368e264f79628e2bda89a545d50cd9925ad50f8e25decc9130164bdb9220c8"
    "f49776d784511e9c4b94305cc2cb3eaf8204e42d31ba8aabd1d296b9ef51035b6df1ab75"
    "89681f0026073ccac6bed5d8bd9235a4bb717b696ce518de4e35b751fa13"
)


class _Pipe(io.IOBase):
    def __init__(self):
        self._other = None
        self.block_reads = False
        self.block_writes = False

        self.write_buffers = []
        self.last_poll_arg = None

    def readinto(self, buf):
        if self.block_reads or len(self._other.write_buffers) == 0:
            return None

        read_buf = self._other.write_buffers[0]
        l = min(len(buf), len(read_buf))
        buf[:l] = read_buf[:l]
        if l == len(read_buf):
            self._other.write_buffers.pop(0)
        else:
            self._other.write_buffers[0] = read_buf[l:]
        return l

    def write(self, buf):
        if self.block_writes:
            return None

        self.write_buffers.append(memoryview(bytes(buf)))
        return len(buf)

    def ioctl(self, request, arg):
        if request == _MP_STREAM_POLL:
            self.last_poll_arg = arg
            ret = 0
            if arg & _MP_STREAM_POLL_RD:
                if not self.block_reads and self._other.write_buffers:
                    ret |= _MP_STREAM_POLL_RD
            if arg & _MP_STREAM_POLL_WR:
                if not self.block_writes:
                    ret |= _MP_STREAM_POLL_WR
            return ret

        elif request == _MP_STREAM_CLOSE:
            return 0

        raise NotImplementedError()

    @classmethod
    def new_pair(cls):
        p1 = cls()
        p2 = cls()
        p1._other = p2
        p2._other = p1
        return p1, p2


def assert_poll(s, i, arg, expected_arg, expected_ret):
    ret = s.ioctl(_MP_STREAM_POLL, arg)
    assert i.last_poll_arg == expected_arg
    i.last_poll_arg = None
    assert ret == expected_ret


def assert_raises(cb, *args, **kwargs):
    try:
        cb(*args, **kwargs)
        raise AssertionError("should have raised")
    except Exception as exc:
        pass


client_io, server_io = _Pipe.new_pair()

client_io.block_reads = True
client_io.block_writes = True
client_sock = ssl.wrap_socket(client_io, do_handshake=False)

server_sock = ssl.wrap_socket(server_io, key=key, cert=cert, server_side=True, do_handshake=False)

# Do a test read, at this point the TLS handshake wants to write,
# so it returns None:
assert client_sock.read(128) is None

# Polling for either read or write actually check if the underlying socket can write:
assert_poll(client_sock, client_io, _MP_STREAM_POLL_RD, _MP_STREAM_POLL_WR, 0)
assert_poll(client_sock, client_io, _MP_STREAM_POLL_WR, _MP_STREAM_POLL_WR, 0)

# Mark the socket as writable, and do another test read:
client_io.block_writes = False
assert client_sock.read(128) is None

# The client wrote the CLIENT_HELLO message
assert len(client_io.write_buffers) == 1

# At this point the TLS handshake wants to read, but we don't know that yet:
assert_poll(client_sock, client_io, _MP_STREAM_POLL_RD, _MP_STREAM_POLL_RD, 0)
assert_poll(client_sock, client_io, _MP_STREAM_POLL_WR, _MP_STREAM_POLL_WR, _MP_STREAM_POLL_WR)

# Do a test write
client_sock.write(b"foo")

# Now we know that we want to read:
assert_poll(client_sock, client_io, _MP_STREAM_POLL_RD, _MP_STREAM_POLL_RD, 0)
assert_poll(client_sock, client_io, _MP_STREAM_POLL_WR, _MP_STREAM_POLL_RD, 0)

# Unblock reads and nudge the two sockets:
client_io.block_reads = False
while server_io.write_buffers or client_io.write_buffers:
    if server_io.write_buffers:
        assert client_sock.read(128) is None
    if client_io.write_buffers:
        assert server_sock.read(128) is None

# At this point, the handshake is done, try writing data:
client_sock.write(b"foo")
assert server_sock.read(3) == b"foo"

# Test reading partial data:
client_sock.write(b"foobar")
assert server_sock.read(3) == b"foo"
server_io.block_reads = True
assert_poll(
    server_sock, server_io, _MP_STREAM_POLL_RD, None, _MP_STREAM_POLL_RD
)  # Did not go to the socket, just consumed buffered data
assert server_sock.read(3) == b"bar"


# Polling on a closed socket errors out:
client_io, _ = _Pipe.new_pair()
client_sock = ssl.wrap_socket(client_io, do_handshake=False)
client_sock.close()
assert_poll(
    client_sock, client_io, _MP_STREAM_POLL_RD, None, _MP_STREAM_POLL_NVAL
)  # Did not go to the socket


# Errors propagates to poll:
client_io, server_io = _Pipe.new_pair()
client_sock = ssl.wrap_socket(client_io, do_handshake=False)

# The server returns garbage:
server_io.write(b"fooba")  # Needs to be exactly 5 bytes

assert_poll(client_sock, client_io, _MP_STREAM_POLL_RD, _MP_STREAM_POLL_RD, _MP_STREAM_POLL_RD)
assert_raises(client_sock.read, 128)
assert_poll(
    client_sock, client_io, _MP_STREAM_POLL_RD, None, _MP_STREAM_POLL_NVAL
)  # Did not go to the socket



from pyb import Pin

p = Pin("X8", Pin.IN)
print(p)
print(p.name())
print(p.pin())
print(p.port())

p = Pin("X8", Pin.IN, Pin.PULL_UP)
p = Pin("X8", Pin.IN, pull=Pin.PULL_UP)
p = Pin("X8", mode=Pin.IN, pull=Pin.PULL_UP)
print(p)
print(p.value())

p.init(p.IN, p.PULL_DOWN)
p.init(p.IN, pull=p.PULL_DOWN)
p.init(mode=p.IN, pull=p.PULL_DOWN)
print(p)
print(p.value())

p.init(p.OUT_PP)
p.low()
print(p.value())
p.high()
print(p.value())
p.value(0)
print(p.value())
p.value(1)
print(p.value())
p.value(False)
print(p.value())
p.value(True)
print(p.value())



"""Subset of cpython tarfile class methods needed to decode tar files."""

import uctypes

# Minimal set of tar header fields for reading.
# http://www.gnu.org/software/tar/manual/html_node/Standard.html
# The "size" entry is 11 (not 12) to implicitly cut off the null terminator.
_TAR_HEADER = {
    "name": (uctypes.ARRAY | 0, uctypes.UINT8 | 100),
    "size": (uctypes.ARRAY | 124, uctypes.UINT8 | 11),
}

DIRTYPE = const("dir")
REGTYPE = const("file")

# Constants for TarInfo.isdir, isreg.
_S_IFMT = const(0o170000)
_S_IFREG = const(0o100000)
_S_IFDIR = const(0o040000)

_BLOCKSIZE = const(512)  # length of processing blocks


def _roundup(val, align):
    return (val + align - 1) & ~(align - 1)


class FileSection:
    def __init__(self, f, content_len, aligned_len):
        self.f = f
        self.content_len = content_len
        self.align = aligned_len - content_len

    def read(self, sz=65536):
        if self.content_len == 0:
            return b""
        if sz > self.content_len:
            sz = self.content_len
        data = self.f.read(sz)
        sz = len(data)
        self.content_len -= sz
        return data

    def readinto(self, buf):
        if self.content_len == 0:
            return 0
        if len(buf) > self.content_len:
            buf = memoryview(buf)[: self.content_len]
        sz = self.f.readinto(buf)
        self.content_len -= sz
        return sz

    def skip(self):
        sz = self.content_len + self.align
        if sz:
            buf = bytearray(16)
            while sz:
                s = min(sz, 16)
                self.f.readinto(buf, s)
                sz -= s


class TarInfo:
    def __init__(self, name=""):
        self.name = name
        self.mode = _S_IFDIR if self.name[-1] == "/" else _S_IFREG

    @property
    def type(self):
        return DIRTYPE if self.isdir() else REGTYPE

    def __str__(self):
        return "TarInfo(%r, %s, %d)" % (self.name, self.type, self.size)

    def isdir(self):
        return (self.mode & _S_IFMT) == _S_IFDIR

    def isreg(self):
        return (self.mode & _S_IFMT) == _S_IFREG


class TarFile:
    def __init__(self, name=None, mode="r", fileobj=None):
        self.subf = None
        self.mode = mode
        self.offset = 0
        if mode == "r":
            if fileobj:
                self.f = fileobj
            else:
                self.f = open(name, "rb")
        else:
            try:
                self._open_write(name=name, mode=mode, fileobj=fileobj)
            except AttributeError:
                raise NotImplementedError("Install tarfile-write")

    def __enter__(self):
        return self

    def __exit__(self, unused_type, unused_value, unused_traceback):
        self.close()

    def next(self):
        if self.subf:
            self.subf.skip()
        buf = self.f.read(_BLOCKSIZE)
        if not buf:
            return None

        h = uctypes.struct(uctypes.addressof(buf), _TAR_HEADER, uctypes.LITTLE_ENDIAN)

        # Empty block means end of archive
        if h.name[0] == 0:
            return None

        # Update the offset once we're sure it's not the run-out.
        self.offset += len(buf)
        d = TarInfo(str(h.name, "utf-8").rstrip("\0"))
        d.size = int(bytes(h.size), 8)
        self.subf = d.subf = FileSection(self.f, d.size, _roundup(d.size, _BLOCKSIZE))
        self.offset += _roundup(d.size, _BLOCKSIZE)
        return d

    def __iter__(self):
        return self

    def __next__(self):
        v = self.next()
        if v is None:
            raise StopIteration
        return v

    def extractfile(self, tarinfo):
        return tarinfo.subf

    def close(self):
        try:
            self._close_write()
        except AttributeError:
            pass
        self.f.close()

    # Add additional methods to support write/append from the tarfile-write package.
    try:
        from .write import _open_write, _close_write, addfile, add
    except ImportError:
        pass



import sys, pkg

# Modules we import are usually added to sys.modules.
print("pkg" in sys.modules)

try:
    from broken.zerodiv import x
except Exception as e:
    print(e.__class__.__name__)

# The broken module we tried to import should not be in sys.modules.
print("broken.zerodiv" in sys.modules)

# If we try to import the module again, the code should
# run again and we should get the same error.
try:
    from broken.zerodiv import x
except Exception as e:
    print(e.__class__.__name__)

# Import a module that successfully imports some other modules
# before importing the problematic module.
try:
    import broken.pkg2_and_zerodiv
except ZeroDivisionError:
    pass

print("pkg2" in sys.modules)
print("pkg2.mod1" in sys.modules)
print("pkg2.mod2" in sys.modules)
print("broken.zerodiv" in sys.modules)
print("broken.pkg2_and_zerodiv" in sys.modules)



import sys

try:
    import uctypes
except ImportError:
    print("SKIP")
    raise SystemExit

if sys.byteorder != "little":
    print("SKIP")
    raise SystemExit


desc = {
    "ptr": (uctypes.PTR | 0, uctypes.UINT8),
    "ptr16": (uctypes.PTR | 0, uctypes.UINT16),
    "ptr2": (uctypes.PTR | 0, {"b": uctypes.UINT8 | 0}),
}

bytes = b"01"

addr = uctypes.addressof(bytes)
buf = addr.to_bytes(uctypes.sizeof(desc), "little")

S = uctypes.struct(uctypes.addressof(buf), desc, uctypes.NATIVE)

print(S.ptr[0])
assert S.ptr[0] == ord("0")
print(S.ptr[1])
assert S.ptr[1] == ord("1")
print(hex(S.ptr16[0]))
assert hex(S.ptr16[0]) == "0x3130"
print(S.ptr2[0].b, S.ptr2[1].b)
print(S.ptr2[0].b, S.ptr2[1].b)
print(hex(S.ptr16[0]))
assert (S.ptr2[0].b, S.ptr2[1].b) == (48, 49)



def f1(**kwargs):
    print(kwargs)

f1()
f1(a=1)

def f2(a, **kwargs):
    print(a, kwargs)

f2(1)
f2(1, b=2)

def f3(a, *vargs, **kwargs):
    print(a, vargs, kwargs)

f3(1)
f3(1, 2)
f3(1, b=2)
f3(1, 2, b=3)

def f4(*vargs, **kwargs):
    print(vargs, kwargs)
f4(*(1, 2))
f4(kw_arg=3)
f4(*(1, 2), kw_arg=3)


# test evaluation order of arguments
def f5(*vargs, **kwargs):
    print(vargs, kwargs)


def print_ret(x):
    print(x)
    return x


f5(*print_ret(["a", "b"]), kw_arg=print_ret(None))



# test memoryview retains pointer to original object/buffer
try:
    memoryview
except:
    print("SKIP")
    raise SystemExit

b = bytearray(10)
m = memoryview(b)[1:]
for i in range(len(m)):
    m[i] = i

# reclaim b, but hopefully not the buffer
b = None
import gc
gc.collect()

# allocate lots of memory
for i in range(100000):
    [42, 42, 42, 42]

# check that the memoryview is still what we want
print(list(m))

# check that creating a memoryview of a memoryview retains the underlying data
m = None
gc.collect()  # cleanup from previous test
m = memoryview(memoryview(bytearray(i for i in range(50)))[5:-5])
print(sum(m), list(m[:10]))
gc.collect()
for i in range(10):
    list(range(10))  # allocate memory to overwrite any reclaimed heap
print(sum(m), list(m[:10]))



# Test generator .throw() with multiple arguments.
# Using multiple arguments is deprecated since CPython 3.12.


def gen():
    try:
        yield 1
    except ValueError as e:
        print("got ValueError from upstream!", repr(e.args))
    yield "str1"
    raise TypeError


def gen2():
    print((yield from gen()))


# Passing None as second argument to throw.
g = gen2()
print(next(g))
print(g.throw(ValueError, None))
try:
    print(next(g))
except TypeError:
    print("got TypeError from downstream!")

# Passing an exception instance as second argument to throw.
g = gen2()
print(next(g))
print(g.throw(ValueError, ValueError(123)))
try:
    print(next(g))
except TypeError:
    print("got TypeError from downstream!")



print("Yep, I got imported.")

try:
    x = const(1)
except NameError:
    # Either running on CPython or MICROPY_COMP_CONST disabled.
    const = lambda x: x


# No const optimisation.
_CNT01 = "CONST01"

# Const assigned to an underscore name. Invisible to MicroPython with
# MICROPY_COMP_CONST enabled.
_CNT02 = const(123)

# Consts assigned to regular name, executed normally.
A123 = const(123)
a123 = const(123)


def dummy():
    return False


def saysomething():
    print("There, I said it.")


def neverexecuted():
    print("Never got here!")


print("Yep, got here")



# Test recv on socket that just accepted a connection

import socket

PORT = 8000


# Server
def instance0():
    multitest.globals(IP=multitest.get_network_ip())
    s = socket.socket()
    s.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
    s.bind(socket.getaddrinfo("0.0.0.0", PORT)[0][-1])
    s.listen(1)
    multitest.next()
    s.accept()
    try:
        print("recv", s.recv(10))  # should raise Errno 107 ENOTCONN
    except OSError as er:
        print(er.errno in (107, 128))
    s.close()


# Client
def instance1():
    multitest.next()
    s = socket.socket()
    s.connect(socket.getaddrinfo(IP, PORT)[0][-1])
    s.send(b"GET / HTTP/1.0\r\n\r\n")
    s.close()



# test machine module

try:
    import machine

    machine.mem8
except:
    print("SKIP")
    raise SystemExit

print(machine.mem8)

try:
    machine.mem16[1]
except ValueError:
    print("ValueError")

try:
    machine.mem16[1] = 1
except ValueError:
    print("ValueError")

try:
    del machine.mem8[0]
except TypeError:
    print("TypeError")

try:
    machine.mem8[0:1]
except TypeError:
    print("TypeError")

try:
    machine.mem8[0:1] = 10
except TypeError:
    print("TypeError")

try:
    machine.mem8["hello"]
except TypeError:
    print("TypeError")

try:
    machine.mem8["hello"] = 10
except TypeError:
    print("TypeError")



b = bytearray(30)
f = open("data/file1", "rb")
print(f.readinto(b))
print(b)
f = open("data/file2", "rb")
print(f.readinto(b))
print(b)

# readinto() on writable file
f = open("data/file1", "ab")
try:
    f.readinto(bytearray(4))
except OSError:
    print("OSError")



import sys


def getmembers(obj, pred=None):
    res = []
    for name in dir(obj):
        val = getattr(obj, name)
        if pred is None or pred(val):
            res.append((name, val))
    res.sort()
    return res


def isfunction(obj):
    return isinstance(obj, type(isfunction))


def isgeneratorfunction(obj):
    return isinstance(obj, type(lambda: (yield)))


def isgenerator(obj):
    return isinstance(obj, type(lambda: (yield)()))


class _Class:
    def meth():
        pass


_Instance = _Class()


def ismethod(obj):
    return isinstance(obj, type(_Instance.meth))


def isclass(obj):
    return isinstance(obj, type)


def ismodule(obj):
    return isinstance(obj, type(sys))


def getargspec(func):
    raise NotImplementedError("This is over-dynamic function, not supported by MicroPython")


def getmodule(obj, _filename=None):
    return None  # Not known


def getmro(cls):
    return [cls]


def getsourcefile(obj):
    return None  # Not known


def getfile(obj):
    return "<unknown>"


def getsource(obj):
    return "<source redacted to save you memory>"


def currentframe():
    return None


def getframeinfo(frame, context=1):
    return ("<unknown>", -1, "<unknown>", [""], 0)



# Tests for the Buffer class included in usb.device.core
#
# The easiest way to run this is using unix port. From the parent usb-device
# directory, run as:
#
# $ micropython -m tests.test_core_buffer
#
import micropython
from usb.device import core

if not hasattr(core.machine, "disable_irq"):
    # Inject a fake machine module which allows testing on the unix port, and as
    # a bonus have tests fail if the buffer allocates inside a critical section.
    class FakeMachine:
        def disable_irq(self):
            return micropython.heap_lock()

        def enable_irq(self, was_locked):
            if not was_locked:
                micropython.heap_unlock()

    core.machine = FakeMachine()


b = core.Buffer(16)

# Check buffer is empty
assert b.readable() == 0
assert b.writable() == 16

# Single write then read
w = b.pend_write()
assert len(w) == 16
w[:8] = b"12345678"
b.finish_write(8)

# Empty again
assert b.readable() == 8
assert b.writable() == 8

r = b.pend_read()
assert len(r) == 8
assert r == b"12345678"
b.finish_read(8)

# Empty buffer again
assert b.readable() == 0
assert b.writable() == 16

# Single write then split reads
b.write(b"abcdefghijklmnop")
assert b.writable() == 0  # full buffer

r = b.pend_read()
assert r == b"abcdefghijklmnop"
b.finish_read(2)

r = b.pend_read()
assert r == b"cdefghijklmnop"
b.finish_read(3)

# write to end of buffer
b.write(b"AB")

r = b.pend_read()
assert r == b"fghijklmnopAB"

# write while a read is pending
b.write(b"XY")

# previous pend_read() memoryview should be the same
assert r == b"fghijklmnopAB"

b.finish_read(4)
r = b.pend_read()
assert r == b"jklmnopABXY"  # four bytes consumed from head, one new byte at tail

# read while a write is pending
w = b.pend_write()
assert len(w) == 5
r = b.pend_read()
assert len(r) == 11
b.finish_read(3)
w[:2] = b"12"
b.finish_write(2)

# Expected final state of buffer
tmp = bytearray(b.readable())
assert b.readinto(tmp) == len(tmp)
assert tmp == b"mnopABXY12"

# Now buffer is empty again
assert b.readable() == 0
assert b.readinto(tmp) == 0
assert b.writable() == 16

print("All Buffer tests passed")



try:
    import binascii
except ImportError:
    print("SKIP")
    raise SystemExit

print(binascii.a2b_base64(b""))
print(binascii.a2b_base64(b"Zg=="))
print(binascii.a2b_base64(b"Zm8="))
print(binascii.a2b_base64(b"Zm9v"))
print(binascii.a2b_base64(b"Zm9vYg=="))
print(binascii.a2b_base64(b"Zm9vYmE="))
print(binascii.a2b_base64(b"Zm9vYmFy"))

print(binascii.a2b_base64(b"AAECAwQFBgc="))
print(binascii.a2b_base64(b"CAkKCwwNDg8="))
print(binascii.a2b_base64(b"f4D/"))
print(binascii.a2b_base64(b"f4D+"))  # convert '+'
print(binascii.a2b_base64(b"MTIzNEFCQ0RhYmNk"))

# Ignore invalid characters and pad sequences
print(binascii.a2b_base64(b"Zm9v\n"))
print(binascii.a2b_base64(b"Zm\x009v\n"))
print(binascii.a2b_base64(b"Zm9v=="))
print(binascii.a2b_base64(b"Zm9v==="))
print(binascii.a2b_base64(b"Zm9v===YmFy"))

try:
    print(binascii.a2b_base64(b"abc"))
except ValueError:
    print("ValueError")
try:
    print(binascii.a2b_base64(b"abcde="))
except ValueError:
    print("ValueError")
try:
    print(binascii.a2b_base64(b"ab*d"))
except ValueError:
    print("ValueError")
try:
    print(binascii.a2b_base64(b"ab=cdef="))
except ValueError:
    print("ValueError")



# test StopIteration interaction with generators

try:
    enumerate, exec
except:
    print("SKIP")
    raise SystemExit


def get_stop_iter_arg(msg, code):
    try:
        exec(code)
        print("FAIL")
    except StopIteration as er:
        print(msg, er.args)


class A:
    def __iter__(self):
        return self

    def __next__(self):
        raise StopIteration(42)


class B:
    def __getitem__(self, index):
        # argument to StopIteration should get ignored
        raise StopIteration(42)


def gen(x):
    return x
    yield


def gen2(x):
    try:
        yield
    except ValueError:
        pass
    return x


get_stop_iter_arg("next", "next(A())")
get_stop_iter_arg("iter", "next(iter(B()))")
get_stop_iter_arg("enumerate", "next(enumerate(A()))")
get_stop_iter_arg("map", "next(map(lambda x:x, A()))")
get_stop_iter_arg("zip", "next(zip(A()))")
g = gen(None)
get_stop_iter_arg("generator0", "next(g)")
get_stop_iter_arg("generator1", "next(g)")
g = gen(42)
get_stop_iter_arg("generator0", "next(g)")
get_stop_iter_arg("generator1", "next(g)")
get_stop_iter_arg("send", "gen(None).send(None)")
get_stop_iter_arg("send", "gen(42).send(None)")
g = gen2(None)
next(g)
get_stop_iter_arg("throw", "g.throw(ValueError)")
g = gen2(42)
next(g)
get_stop_iter_arg("throw", "g.throw(ValueError)")



# test 32-bit long arguments and return types for ffi functions

import struct

import ffi_int_base

if struct.calcsize("l") != 4:
    print("SKIP")
    raise SystemExit

ffi_int_base.test(
    [
        ("l", "fli"),
        ("L", "flu"),
    ]
)



"""JSON token scanner
"""
import re

try:
    from _json import make_scanner as c_make_scanner
except ImportError:
    c_make_scanner = None

__all__ = ["make_scanner"]

NUMBER_RE = re.compile(
    r"(-?(?:0|[1-9]\d*))(\.\d+)?([eE][-+]?\d+)?", (re.VERBOSE | re.MULTILINE | re.DOTALL)
)


def py_make_scanner(context):
    parse_object = context.parse_object
    parse_array = context.parse_array
    parse_string = context.parse_string
    match_number = NUMBER_RE.match
    strict = context.strict
    parse_float = context.parse_float
    parse_int = context.parse_int
    parse_constant = context.parse_constant
    object_hook = context.object_hook
    object_pairs_hook = context.object_pairs_hook
    memo = context.memo

    def _scan_once(string, idx):
        try:
            nextchar = string[idx]
        except IndexError:
            raise StopIteration

        if nextchar == '"':
            return parse_string(string, idx + 1, strict)
        elif nextchar == "{":
            return parse_object(
                (string, idx + 1), strict, _scan_once, object_hook, object_pairs_hook, memo
            )
        elif nextchar == "[":
            return parse_array((string, idx + 1), _scan_once)
        elif nextchar == "n" and string[idx : idx + 4] == "null":
            return None, idx + 4
        elif nextchar == "t" and string[idx : idx + 4] == "true":
            return True, idx + 4
        elif nextchar == "f" and string[idx : idx + 5] == "false":
            return False, idx + 5

        m = match_number(string, idx)
        if m is not None:
            integer, frac, exp = m.groups()
            if frac or exp:
                res = parse_float(integer + (frac or "") + (exp or ""))
            else:
                res = parse_int(integer)
            return res, m.end()
        elif nextchar == "N" and string[idx : idx + 3] == "NaN":
            return parse_constant("NaN"), idx + 3
        elif nextchar == "I" and string[idx : idx + 8] == "Infinity":
            return parse_constant("Infinity"), idx + 8
        elif nextchar == "-" and string[idx : idx + 9] == "-Infinity":
            return parse_constant("-Infinity"), idx + 9
        else:
            raise StopIteration

    def scan_once(string, idx):
        try:
            return _scan_once(string, idx)
        finally:
            memo.clear()

    return _scan_once


make_scanner = c_make_scanner or py_make_scanner



# test that socket.accept() on a socket with timeout raises ETIMEDOUT

import errno, socket

try:
    socket.socket.settimeout
except AttributeError:
    print("SKIP")
    raise SystemExit

s = socket.socket()
s.bind(socket.getaddrinfo("127.0.0.1", 8123)[0][-1])
s.settimeout(1)
s.listen(1)
try:
    s.accept()
except OSError as er:
    print(er.errno in (errno.ETIMEDOUT, "timed out"))  # CPython uses a string instead of errno
s.close()



"""
General functions for HTML manipulation.
"""


_escape_map = {ord("&"): "&amp;", ord("<"): "&lt;", ord(">"): "&gt;"}
_escape_map_full = {
    ord("&"): "&amp;",
    ord("<"): "&lt;",
    ord(">"): "&gt;",
    ord('"'): "&quot;",
    ord("'"): "&#x27;",
}

# NB: this is a candidate for a bytes/string polymorphic interface


def escape(s, quote=True):
    """
    Replace special characters "&", "<" and ">" to HTML-safe sequences.
    If the optional flag quote is true (the default), the quotation mark
    characters, both double quote (") and single quote (') characters are also
    translated.
    """
    import string

    if quote:
        return string.translate(s, _escape_map_full)
    return string.translate(s, _escape_map)



# test that socket.connect() on a non-blocking socket raises EINPROGRESS

import socket, errno


def test(peer_addr):
    s = socket.socket()
    s.setblocking(False)
    try:
        s.connect(peer_addr)
    except OSError as er:
        print(er.errno == errno.EINPROGRESS)
    s.close()


if __name__ == "__main__":
    test(socket.getaddrinfo("micropython.org", 80)[0][-1])



# MicroPython USB HID custom Keypad example
#
# This example demonstrates creating a custom HID device with its own
# HID descriptor, in this case for a USB number keypad.
#
# For higher level examples that require less code to use, see mouse_example.py
# and keyboard_example.py
#
# To run this example:
#
# 1. Make sure `usb-device-hid` is installed via: mpremote mip install usb-device-hid
#
# 2. Run the example via: mpremote run hid_custom_keypad_example.py
#
# 3. mpremote will exit with an error after the previous step, because when the
#    example runs the existing USB device disconnects and then re-enumerates with
#    the custom HID interface present. At this point, the example is running.
#
# 4. To see output from the example, re-connect: mpremote connect PORTNAME
#
# MIT license; Copyright (c) 2023 Dave Wickham, 2023-2024 Angus Gratton
from micropython import const
import time
import usb.device
from usb.device.hid import HIDInterface

_INTERFACE_PROTOCOL_KEYBOARD = const(0x01)


def keypad_example():
    k = KeypadInterface()

    usb.device.get().init(k, builtin_driver=True)

    while not k.is_open():
        time.sleep_ms(100)

    while True:
        time.sleep(2)
        print("Press NumLock...")
        k.send_key("<NumLock>")
        time.sleep_ms(100)
        k.send_key()
        time.sleep(1)
        # continue
        print("Press ...")
        for _ in range(3):
            time.sleep(0.1)
            k.send_key(".")
            time.sleep(0.1)
            k.send_key()
        print("Starting again...")


class KeypadInterface(HIDInterface):
    # Very basic synchronous USB keypad HID interface

    def __init__(self):
        super().__init__(
            _KEYPAD_REPORT_DESC,
            set_report_buf=bytearray(1),
            protocol=_INTERFACE_PROTOCOL_KEYBOARD,
            interface_str="MicroPython Keypad",
        )
        self.numlock = False

    def on_set_report(self, report_data, _report_id, _report_type):
        report = report_data[0]
        b = bool(report & 1)
        if b != self.numlock:
            print("Numlock: ", b)
            self.numlock = b

    def send_key(self, key=None):
        if key is None:
            self.send_report(b"\x00")
        else:
            self.send_report(_key_to_id(key).to_bytes(1, "big"))


# See HID Usages and Descriptions 1.4, section 10 Keyboard/Keypad Page (0x07)
#
# This keypad example has a contiguous series of keys (KEYPAD_KEY_IDS) starting
# from the NumLock/Clear keypad key (0x53), but you can send any Key IDs from
# the table in the HID Usages specification.
_KEYPAD_KEY_OFFS = const(0x53)

_KEYPAD_KEY_IDS = [
    "<NumLock>",
    "/",
    "*",
    "-",
    "+",
    "<Enter>",
    "1",
    "2",
    "3",
    "4",
    "5",
    "6",
    "7",
    "8",
    "9",
    "0",
    ".",
]


def _key_to_id(key):
    # This is a little slower than making a dict for lookup, but uses
    # less memory and O(n) can be fast enough when n is small.
    return _KEYPAD_KEY_IDS.index(key) + _KEYPAD_KEY_OFFS


# HID Report descriptor for a numeric keypad
#
# fmt: off
_KEYPAD_REPORT_DESC = (
    b'\x05\x01'  # Usage Page (Generic Desktop)
        b'\x09\x07'  # Usage (Keypad)
    b'\xA1\x01'  # Collection (Application)
        b'\x05\x07'  # Usage Page (Keypad)
            b'\x19\x00'  # Usage Minimum (0)
            b'\x29\xFF'  # Usage Maximum (ff)
            b'\x15\x00'  # Logical Minimum (0)
            b'\x25\xFF'  # Logical Maximum (ff)
            b'\x95\x01'  # Report Count (1),
            b'\x75\x08'  # Report Size (8),
            b'\x81\x00'  # Input (Data, Array, Absolute)
        b'\x05\x08'  # Usage page (LEDs)
            b'\x19\x01'  # Usage Minimum (1)
            b'\x29\x01'  # Usage Maximum (1),
            b'\x95\x01'  # Report Count (1),
            b'\x75\x01'  # Report Size (1),
            b'\x91\x02'  # Output (Data, Variable, Absolute)
            b'\x95\x01'  # Report Count (1),
            b'\x75\x07'  # Report Size (7),
            b'\x91\x01'  # Output (Constant) - padding bits
    b'\xC0'  # End Collection
)
# fmt: on


keypad_example()



# This list of package files doesn't include task.py because that's provided
# by the C module.
package(
    "asyncio",
    (
        "__init__.py",
        "core.py",
        "event.py",
        "funcs.py",
        "lock.py",
        "stream.py",
    ),
    base_path="..",
    opt=3,
)

# Backwards-compatible uasyncio module.
module("uasyncio.py", opt=3)



# Simple test of a UDP server and client transferring data

import socket

NUM_NEW_SOCKETS = 4
NUM_TRANSFERS = 10
TOTAL_PACKETS = NUM_NEW_SOCKETS * NUM_TRANSFERS
# If more than 75% of packets are lost, the test fails.
PACKET_LOSS_THRESH = 0.75 * TOTAL_PACKETS
PORT = 8000


def print_stats(seq):
    if (TOTAL_PACKETS - seq) > PACKET_LOSS_THRESH:
        print(
            "packet loss %.1f%% %d/%d"
            % (((TOTAL_PACKETS - seq) / TOTAL_PACKETS * 100), seq, TOTAL_PACKETS)
        )
    else:
        print("pass")


# Server
def instance0():
    seq = 0
    multitest.globals(IP=multitest.get_network_ip())
    multitest.next()
    for i in range(NUM_NEW_SOCKETS):
        s = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
        s.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
        s.bind(socket.getaddrinfo("0.0.0.0", PORT + i)[0][-1])
        s.settimeout(0.250)
        multitest.broadcast("server ready")
        for j in range(NUM_TRANSFERS):
            try:
                data, addr = s.recvfrom(1000)
            except:
                continue
            if int(data) == seq:
                if seq < (TOTAL_PACKETS - PACKET_LOSS_THRESH):
                    print(seq)
                seq += 1
                s.sendto(b"%d" % (seq), addr)
        s.close()

    print_stats(seq)


# Client
def instance1():
    seq = 0
    multitest.next()
    for i in range(NUM_NEW_SOCKETS):
        ai = socket.getaddrinfo(IP, PORT + i)[0][-1]
        s = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
        s.settimeout(0.250)
        multitest.wait("server ready")
        for j in range(NUM_TRANSFERS):
            s.sendto(b"%d" % (seq), ai)
            try:
                data, addr = s.recvfrom(1000)
            except:
                continue
            if int(data) == seq + 1:
                if seq < (TOTAL_PACKETS - PACKET_LOSS_THRESH):
                    print(seq)
                seq += 1
        s.close()

    print_stats(seq)



# test waiting within async with enter/exit functions

import sys
if sys.implementation.name == 'micropython':
    # uPy allows normal generators to be awaitables
    coroutine = lambda f: f
else:
    import types
    coroutine = types.coroutine

@coroutine
def f(x):
    print('f start:', x)
    yield x + 1
    yield x + 2
    return x + 3

class AContext:
    async def __aenter__(self):
        print('enter')
        print('f returned:', await f(10))
    async def __aexit__(self, exc_type, exc, tb):
        print('exit', exc_type, exc)
        print('f returned:', await f(20))

async def coro():
    async with AContext():
        print('body start')
        print('body f returned:', await f(30))
        print('body end')

o = coro()
try:
    while True:
        print('coro yielded:', o.send(None))
except StopIteration:
    print('finished')
